import sys,ast
import tokenize
# sys.path.append("/Users/zhangzejunzhangzejun/PycharmProjects/pythonProject/python-ast-explorer-master/extract_transform_complicate_code_new/comprehension/")

import extract_compli_for_comprehension_logic
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
from RefactoringIdioms.transform_c_s import transform_comprehension_dict_compli_to_simple
from tokenize import tokenize
from io import BytesIO
# from pathos.multiprocessing import ProcessingPool as newPool
import traceback
'''
get comprehension node
Block=> Assign;...; For
For=>”for” ( For| If|If-else|body)
If=>”if” (body| For| If| If-else)
If-else=>”if” body “else” (body| If-else)
body=> (Assign|Call)*;Append*
并且里面只有一个写变量
for

    b.append()
    
    a.append(b)

'''

const_empty_list=[" = dict()"," = {}"]


def get_var(one_body):
    var_name=None
    if isinstance(one_body, ast.Assign):
        targets = one_body.targets
        if len(targets) > 1:
            return False

        left_node = targets[0]
        if isinstance(left_node, ast.Subscript):
            var_name=ast.unparse(left_node.value)
    return var_name



def whether_fun_is_append(one_body,assign_block_list):
    #print("one_body: ",ast.unparse(one_body),one_body.__dict__)
    if isinstance(one_body, ast.Assign):
        #print("come here")
        targets=one_body.targets
        if len(targets)>1:
            return False
        left_node=targets[0]
        if isinstance(left_node, ast.Subscript):
            assign_block_list.append([one_body])
            return True
    return False
def is_use_var(next_child, vars):
    s = ast.unparse(next_child)
    if s != list(vars)[0]:
        g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
        for toknum, to_child, _, _, _ in g:
            if to_child.strip() == list(vars)[0]:
                return 1
    return 0
def whether_first_var_is_empty_assign(tree,for_node,vars,const_empty_list=["[]"]):
    assign_stmt=None
    assign_stmt_lineno=None
    flag=0
    # start_lineno=0
    # print("the var: ",vars)

    def visit_single_vars(target, list_vars):
        # print(">>>>>>>target: ",target.__dict__)
        if isinstance(target, (ast.Name)):
            list_vars.append(ast.unparse(target))
        elif isinstance(target, ast.Subscript):
            list_vars.append(ast.unparse(target))
            visit_single_vars(target.value, list_vars)
        elif isinstance(target, ast.Attribute):
            # print("attr: ", ast.unparse(target))
            list_vars.append(ast.unparse(target))
            visit_single_vars(target.value, list_vars)

    def get_time_var(vars):
        all_var_list = []
        for ind,e_var in enumerate(ast.walk(ast.parse(list(vars)[0]))):

            if ast.unparse(e_var)==list(vars)[0] and isinstance(e_var,(ast.Subscript,ast.Attribute,ast.Name)):
                # print("var: ",e_var,ast.unparse(e_var))
                visit_single_vars(e_var, all_var_list)
                break
        return all_var_list
    def whether_contain_var(node,vars,time_var_list):
        count=0
        s = ast.unparse(node)
        if s != list(vars)[0]:
            g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string

            for toknum, child, _, _, _ in g:
                if child in time_var_list:
                    count+=1
                    # print("yes use more than one time: ", s)
        return count

    time_var_list = get_time_var(vars)
    remove_ass_flag = 0
    time=whether_contain_var(for_node,vars, time_var_list)

    append_time = 0
    for child in ast.walk(for_node):

        # if ast.unparse(child)==list(vars)[0]:
        #     time+=1
        #     if time-append_time > 0:
        #         print("come here the for node use var more than one time ",list(vars)[0])
        #         flag = 1
        #         break
        if isinstance(child,ast.Assign):
            a = []
            append_time += whether_fun_is_append(child, a)
        if isinstance(child, ast.Call):

            def is_call_use_vars(tree, fun_name, time_var_list):
                # print(">>>>>>.come is_call_use_vars: ", fun_name)
                ana_py_fun = ast_util.Fun_Analyzer()
                ana_py_fun.visit(tree)
                for tree_fun, class_name in ana_py_fun.func_def_list:
                    if hasattr(tree_fun, "name"):
                        each_fun_name = tree_fun.name
                        # print(">>>>>>each_fun_name: ", fun_name)
                        if each_fun_name == fun_name:
                            for child in ast.walk(tree_fun):
                                if ast.unparse(child) in time_var_list:
                                    # print("yes the call use vars: ", each_fun_name)
                                    return 1
                return 0

            fun_name = ast.unparse(child.func).split(".")[-1]
            call_use_var_flag = is_call_use_vars(tree, fun_name, time_var_list)
            if call_use_var_flag:
                flag = 1
                return flag, assign_stmt_lineno, assign_stmt,remove_ass_flag
        # time += whether_contain_var(child, vars)
    if time - append_time > 0:
            # print("come here the for node use var more than one time ", list(vars)[0])
            flag = 1
            return flag, assign_stmt_lineno, assign_stmt,remove_ass_flag
            # break
    for_node_block_record=None
    for node in ast.walk(tree):


        if hasattr(node,"lineno") and node.lineno==for_node.lineno:
            break
        if hasattr(node, 'body') and isinstance(node.body, list) and \
                ((hasattr(node, 'lineno') and node.lineno < for_node.lineno <= node.end_lineno) or (
                        hasattr(node.body[0], 'lineno') and node.body[0].lineno < for_node.lineno < node.body[
                    -1].end_lineno)):
            for child in node.body:
                # print("come here node: ",child,ast.unparse(child),isinstance(child,ast.Expr))

                if isinstance(child, ast.Assign):
                    if ast.unparse(child).strip().split("=")[0].strip() == list(vars)[0] and ast.unparse(child.value) in const_empty_list:
                        assign_stmt = child
                        assign_stmt_lineno = child.lineno
                        ass_block_end_lineno=node.end_lineno if hasattr(node,'lineno') else node.body[0].lineno
                        # if ass_block_end_lineno < for_node.lineno:
                        #     remove_ass_flag = 0

                        for_node_block_record = node
                        flag = 2
    if for_node_block_record:
        # print(">>>>>>>come here for_node_block_record:\n ",ast.unparse(for_node_block_record))
        for child in ast.iter_child_nodes(for_node_block_record):  # .body:
            if child == for_node:
                remove_ass_flag = 1
        for node in ast.walk(for_node_block_record):
            # 在ass和for_node之间不可以使用变量vars
            if hasattr(node, "end_lineno") and hasattr(node, "lineno"):
                if node != assign_stmt and node.end_lineno < for_node.lineno and node.lineno > assign_stmt_lineno:
                    s = ast.unparse(node)

                    if s != list(vars)[0]:
                        g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
                        for toknum, child, _, _, _ in g:
                            if child.strip() in time_var_list:  # list(vars)[0]:
                                flag = 1
                                # print("yes use more than one time: ",vars,s,"\n>>>>>>>>>",ast.unparse(for_node))
                                return flag, assign_stmt_lineno, assign_stmt, remove_ass_flag
            if isinstance(node, (ast.For, ast.While,
                                 ast.AsyncFor)) and for_node.lineno > node.lineno > assign_stmt.lineno and node.end_lineno >= for_node.end_lineno:
                # print(">>>>>>>>come here ")
                return 1, assign_stmt_lineno, assign_stmt, remove_ass_flag

    else:
        return 1, assign_stmt_lineno, assign_stmt, remove_ass_flag

            # if s!=list(vars)[0]:
            #     g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
            #     for toknum, child, _, _, _ in g:
            #         if child.strip() == list(vars)[0]:
            #             flag = 1
            #             # print("yes use more than one time: ",vars,s,"\n>>>>>>>>>",ast.unparse(for_node))
            #             break
            #     if flag==1:
            #         break

    return flag,assign_stmt_lineno,assign_stmt,remove_ass_flag
'''
def whether_first_var_is_empty_assign(tree,for_node,vars,const_empty_list=["[]"]):
    assign_stmt=None
    assign_stmt_lineno=None
    flag=0
    # start_lineno=0
    # print("the var: ",vars)


    def whether_contain_var(node,vars):
        count=0
        s = ast.unparse(node)
        if s != list(vars)[0]:
            g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string

            for toknum, child, _, _, _ in g:
                if child == list(vars)[0]:
                    count+=1
                    # print("yes use more than one time: ", s)
        return count

    time=whether_contain_var(for_node, vars)
    append_time = 0
    for child in ast.walk(for_node):

        # if ast.unparse(child)==list(vars)[0]:
        #     time+=1
        #     if time-append_time > 0:
        #         print("come here the for node use var more than one time ",list(vars)[0])
        #         flag = 1
        #         break
        if isinstance(child,ast.Assign):
            a = []
            append_time += whether_fun_is_append(child, a)
        if isinstance(child, ast.Call):



            def is_call_use_vars(tree, fun_name, vars):
                # print(">>>>>>.come is_call_use_vars: ", fun_name)
                ana_py_fun = ast_util.Fun_Analyzer()
                ana_py_fun.visit(tree)
                for tree_fun, class_name in ana_py_fun.func_def_list:
                    if hasattr(tree_fun,"name"):
                        each_fun_name = tree_fun.name
                        # print(">>>>>>each_fun_name: ", fun_name)
                        if each_fun_name == fun_name:
                            for child in ast.walk(tree_fun):
                                if ast.unparse(child) == list(vars)[0]:
                                    # print("yes the call use vars: ", each_fun_name)
                                    return 1
                return 0

            fun_name = ast.unparse(child.func).split(".")[-1]
            call_use_var_flag = is_call_use_vars(tree, fun_name, vars)
            if call_use_var_flag:
                flag = 1
                return flag, assign_stmt_lineno, assign_stmt
        # time += whether_contain_var(child, vars)
    if time - append_time > 0:
            # print("come here the for node use var more than one time ", list(vars)[0])
            flag = 1
            return flag, assign_stmt_lineno, assign_stmt
            # break
    for node in ast.walk(tree):
        if not hasattr(node,"lineno"):
            continue

        if node.lineno==for_node.lineno:
            break
        if isinstance(node, ast.Assign):
            # print("the assign: ",ast.unparse(node),ast.unparse(node).strip().split("=")[0].strip())
            if ast.unparse(node).strip().split("=")[0].strip()==list(vars)[0] and ast.unparse(node.value) in const_empty_list:
                assign_stmt=node
                assign_stmt_lineno=node.lineno
                flag = 2
        if assign_stmt_lineno and node.end_lineno<for_node.lineno and node.lineno>assign_stmt_lineno :
            s=ast.unparse(node)

            if s!=list(vars)[0]:
                g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
                for toknum, child, _, _, _ in g:
                    if child.strip() == list(vars)[0]:
                        flag = 1
                        # print("yes use more than one time: ",vars,s,"\n>>>>>>>>>",ast.unparse(for_node))
                        break
                if flag==1:
                    break
    return flag,assign_stmt_lineno,assign_stmt
'''
def else_traverse(if_body,assign_block_list):
    else_body_list = if_body.orelse
    if len(else_body_list) > 1:
        return False
    else:
        else_body_one = else_body_list[0]
        if isinstance(else_body_one, ast.If):
            or_else = else_body_one.orelse
            if not or_else:
                return False
            else:
                return if_else_traverse(else_body_one,assign_block_list)
        return whether_fun_is_append(else_body_one,assign_block_list)



def if_else_traverse(if_body,assign_block_list):
    if_body_list=if_body.body
    # else_body_list=if_body.orelse
    if len(if_body_list)==1:
        if_body_flag =whether_fun_is_append(if_body_list[0],assign_block_list)
    else:
        return False
    #print("if body flag: ",if_body_flag)

    return if_body_flag and else_traverse(if_body,assign_block_list)

def if_traverse(one_body,assign_block_list):
    orelse = one_body.orelse
    if_body=one_body.body
    if not orelse:
        return for_traverse(one_body,assign_block_list)
    else:
        #print("it is if-else node: ", ast.unparse(one_body))
        return if_else_traverse(one_body,assign_block_list)


def for_traverse(node,assign_block_list):
    for_body_list = node.body
    if isinstance(node, ast.For) and node.orelse:
        #print("come here")
        return False
    if len(for_body_list) == 1:
        one_body=for_body_list[0]
        if isinstance(one_body, ast.For):
            return for_traverse(one_body,assign_block_list)
        elif isinstance(one_body, ast.If):
            #print("it is if node")
            return if_traverse(one_body,assign_block_list)

        elif isinstance(one_body, ast.Assign):
            #print("it is one stmt")
            return whether_fun_is_append(one_body,assign_block_list)
        else:

            #print("the for/if sentences has other stmts so cannot transform such as try stmt", ast.unparse(one_body),type(one_body))
            return False
    else:
        return False
'''
beg1 end1
beg2 end2
'''


def get_complicated_for_comprehen_code_list(tree,content):

    #tree = ast.parse(code1)
    code_index_start_end_list = []

    start_lineno=1
    #print("tree: ",tree.body[0].__dict__)
    start_lineno=tree.body[0].lineno
    #print("start_lineno: ",start_lineno)
    #code1 = "\n".join(content.split("\n")[start_lineno-1:])

    for ind_line,node in enumerate(ast.walk(tree)):

        if isinstance(node, ast.For):
            #print("node_line: ",ast.unparse(node),node.lineno, node.end_lineno)
            # if node.lineno!=1674:
            #     continue

            assign_block_list=[]
            #print("it is what: ",node.lineno)
            if for_traverse(node,assign_block_list):
                #print("maybe comrephension")
                vars = set([])
                #print("maybe comrephension: ",node.lineno,ast.unparse(assign_block_list[0][0]))
                for each_block in assign_block_list:
                    var_name=get_var(each_block[0])
                    vars.add(var_name)
                if len(vars)!=1:
                    continue
                # print("vars: ",vars,ind_line,start_lineno)
                # print("node: ", ast.unparse(node))
                flag,assign_stmt_lineno,assign_stmt,remove_ass_flag=whether_first_var_is_empty_assign(tree, node, vars,["dict()","{}"])
                # print("flag: ", flag)
                if flag!=2:
                    # print("flag: ")#, ast.unparse(node)
                    continue

                assign_block_list_str = []
                for one_block in assign_block_list:
                    one_block_str = []
                    for e in one_block:
                        one_block_str.append(ast.unparse(e))
                    assign_block_list_str.append(one_block_str)
                code_index_start_end_list.append(
                        [node, assign_stmt,remove_ass_flag])#, node.lineno, node.end_lineno, assign_stmt_lineno, assign_block_list_str])

                # print(">>>>>>>>>>>>>>>>>>>>>>it is comprehension code1\n", remove_ass_flag,node.lineno, node.end_lineno,assign_stmt_lineno,ast.unparse(node))
                # code_index_start_end_list.append([ast.unparse(node), node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])
    #print("len: ", len(code_index_start_end_list))
    code_index_start_end_list = extract_compli_for_comprehension_logic.filter_overlap(code_index_start_end_list)
    # code_index_start_end_list = filter_overlap(code_index_start_end_list)
    # print("len: ",len(code_index_start_end_list))
    return code_index_start_end_list
def get_dict_compreh(content):
    code_pair_list=[]
    # print(content)

    try:

        file_tree = ast.parse(content)
        ana_py = ast_util.Fun_Analyzer()
        ana_py.visit(file_tree)
        # print("come here",ana_py.func_def_list)
        for tree, class_name in ana_py.func_def_list:
            # print("come method",ast.unparse(tree))
            try:
                me_name = tree.name
            except:
                me_name = ""
            new_code_list = get_complicated_for_comprehen_code_list(tree, content)

            for ind, (for_node, assign_node, remove_ass_flag) in enumerate(new_code_list):
                new_code= transform_comprehension_dict_compli_to_simple.transform(for_node, assign_node)
                complete_new_code=ast.unparse(new_code) if remove_ass_flag else ast.unparse(assign_node) + "\n"+ast.unparse(new_code)
                assign_node_content = ast.get_source_segment(content, assign_node)
                for_node_content = ast.get_source_segment(content, for_node)
                if remove_ass_flag:
                    line_list = []
                    line_list.append([(assign_node.lineno,assign_node.col_offset), (assign_node.end_lineno,assign_node.end_col_offset)])
                    line_list.append([(for_node.lineno,for_node.col_offset), (for_node.end_lineno,for_node.end_col_offset)])
                    code_pair_list.append([class_name, me_name, [assign_node_content, for_node_content],
                                           ['', ast.unparse(new_code)],
                                           line_list])
                else:
                    line_list = []
                    line_list.append([(for_node.lineno,for_node.col_offset), (for_node.end_lineno,for_node.end_col_offset)])
                    code_pair_list.append(
                        [class_name, me_name, [for_node_content], [ast.unparse(new_code)], line_list])


                # if remove_ass_flag:
                #     line_list = []
                #     line_list.append([assign_node.lineno, assign_node.end_lineno])
                #     line_list.append([for_node.lineno, for_node.end_lineno])
                #     code_pair_list.append([class_name, me_name, [ast.unparse(assign_node), ast.unparse(for_node)],
                #                            ['', ast.unparse(new_code)],
                #                            line_list])
                # else:
                #     line_list = []
                #     line_list.append([for_node.lineno, for_node.end_lineno])
                #     code_pair_list.append([class_name,me_name,[ast.unparse(for_node)], [ast.unparse(new_code)],line_list])
                # #
                # line_list = []
                # line_list.append([assign_node.lineno, assign_node.end_lineno])
                # line_list.append([for_node.lineno, for_node.end_lineno])
                # code_pair_list.append([class_name,me_name,ast.unparse(assign_node) + "\n" + ast.unparse(for_node), complete_new_code,line_list])
        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list

def get_part_dict_compreh(content):
    code_pair_list=[]
    # print(content)

    try:

        tree = ast.parse(content)
        # ana_py = ast_util.Fun_Analyzer()
        # ana_py.visit(file_tree)
        # print("come here",ana_py.func_def_list)
        if 1:#for tree, class_name in ana_py.func_def_list:
            # print("come method",ast.unparse(tree))
            try:
                me_name = tree.name
            except:
                me_name = ""
            new_code_list = get_complicated_for_comprehen_code_list(tree, content)

            for ind, (for_node, assign_node, remove_ass_flag) in enumerate(new_code_list):
                new_code= transform_comprehension_dict_compli_to_simple.transform(for_node, assign_node)
                complete_new_code=ast.unparse(new_code) if remove_ass_flag else ast.unparse(assign_node) + "\n"+ast.unparse(new_code)
                assign_node_content = ast.get_source_segment(content, assign_node)
                for_node_content = ast.get_source_segment(content, for_node)
                if remove_ass_flag:
                    line_list = []
                    line_list.append([(assign_node.lineno,assign_node.col_offset), (assign_node.end_lineno,assign_node.end_col_offset)])
                    line_list.append([(for_node.lineno,for_node.col_offset), (for_node.end_lineno,for_node.end_col_offset)])
                    code_pair_list.append(['', me_name, [assign_node_content, for_node_content],
                                           ['', ast.unparse(new_code)],
                                           line_list])
                else:
                    line_list = []
                    line_list.append([(for_node.lineno,for_node.col_offset), (for_node.end_lineno,for_node.end_col_offset)])
                    code_pair_list.append(
                        ['', me_name, [for_node_content], [ast.unparse(new_code)], line_list])


                # if remove_ass_flag:
                #     line_list = []
                #     line_list.append([assign_node.lineno, assign_node.end_lineno])
                #     line_list.append([for_node.lineno, for_node.end_lineno])
                #     code_pair_list.append([class_name, me_name, [ast.unparse(assign_node), ast.unparse(for_node)],
                #                            ['', ast.unparse(new_code)],
                #                            line_list])
                # else:
                #     line_list = []
                #     line_list.append([for_node.lineno, for_node.end_lineno])
                #     code_pair_list.append([class_name,me_name,[ast.unparse(for_node)], [ast.unparse(new_code)],line_list])
                # #
                # line_list = []
                # line_list.append([assign_node.lineno, assign_node.end_lineno])
                # line_list.append([for_node.lineno, for_node.end_lineno])
                # code_pair_list.append([class_name,me_name,ast.unparse(assign_node) + "\n" + ast.unparse(for_node), complete_new_code,line_list])
        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list
# def save_one_repo(repo_name):
#     count=0
#     one_repo_chained_comparison_code_list = []
#     dict_file = dict()
#     for file_info in dict_repo_file_python[repo_name]:
#         file_path = file_info["file_path"]
#         file_html = file_info["file_html"]
#         # if file_html != "https://github.com/networkx/networkx/tree/master/networkx/algorithms/connectivity/kcomponents.py":#"https://github.com/IDSIA/sacred/tree/master/sacred/observers/mongo.py":  # "https://github.com/cloud-custodian/cloud-custodian/tree/master/c7n/provider.py":#"https://github.com/networkx/networkx/tree/master/networkx/generators/joint_degree_seq.py":#"https://github.com/bndr/pipreqs/tree/master/pipreqs/pipreqs.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/networkx/networkx/tree/master/networkx/readwrite/json_graph/adjacency.py":##"https://github.com/microsoft/nni/tree/master/nni/tools/nnictl/nnictl_utils.py":#"https://github.com/networkx/networkx/tree/master/networkx/readwrite/json_graph/adjacency.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/aws/aws-cli/tree/master/awscli/customizations/s3/subcommands.py":#"https://github.com/nccgroup/ScoutSuite/tree/master/ScoutSuite/providers/aws/utils.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":#"https://github.com/localstack/localstack/tree/master/localstack/services/awslambda/lambda_api.py":#"https://github.com/localstack/localstack/tree/master/localstack/utils/common.py":#"https://github.com/aws/aws-cli/tree/master/awscli/compat.py":#"https://github.com/spulec/moto/tree/master/moto/config/models.py":#"https://github.com/cookiecutter/cookiecutter/tree/master/cookiecutter/hooks.py":#"https://github.com/yt-dlp/yt-dlp/tree/master/yt_dlp/utils.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/OmkarPathak/pygorithm/tree/master/pygorithm/greedy_algorithm/fractional_knapsack.py":#"https://github.com/OmkarPathak/pygorithm/tree/master/pygorithm/strings/anagram.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/cloudtools/troposphere/tree/master/troposphere/validators.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/extensions/nesting.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/sympy/sympy/tree/master/sympy/crypto/crypto.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/sympy/sympy/tree/master/sympy/crypto/crypto.py":#"https://github.com/yt-dlp/yt-dlp/tree/master/yt_dlp/utils.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/extensions/nesting.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":  # "https://github.com/pymc-devs/pymc/tree/master/pymc/sampling.py":#"https://github.com/smicallef/spiderfoot/tree/master//sflib.py":#"https://github.com/smicallef/spiderfoot/tree/master//sfwebui.py":#
#         #     continue
#         # if file_html != "https://github.com/networkx/networkx/tree/master/networkx/generators/joint_degree_seq.py":  #"https://github.com/cloud-custodian/cloud-custodian/tree/master/c7n/provider.py":  # "https://github.com/networkx/networkx/tree/master/networkx/generators/joint_degree_seq.py":#"https://github.com/bndr/pipreqs/tree/master/pipreqs/pipreqs.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/networkx/networkx/tree/master/networkx/readwrite/json_graph/adjacency.py":##"https://github.com/microsoft/nni/tree/master/nni/tools/nnictl/nnictl_utils.py":#"https://github.com/networkx/networkx/tree/master/networkx/readwrite/json_graph/adjacency.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/aws/aws-cli/tree/master/awscli/customizations/s3/subcommands.py":#"https://github.com/nccgroup/ScoutSuite/tree/master/ScoutSuite/providers/aws/utils.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":#"https://github.com/localstack/localstack/tree/master/localstack/services/awslambda/lambda_api.py":#"https://github.com/localstack/localstack/tree/master/localstack/utils/common.py":#"https://github.com/aws/aws-cli/tree/master/awscli/compat.py":#"https://github.com/spulec/moto/tree/master/moto/config/models.py":#"https://github.com/cookiecutter/cookiecutter/tree/master/cookiecutter/hooks.py":#"https://github.com/yt-dlp/yt-dlp/tree/master/yt_dlp/utils.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/OmkarPathak/pygorithm/tree/master/pygorithm/greedy_algorithm/fractional_knapsack.py":#"https://github.com/OmkarPathak/pygorithm/tree/master/pygorithm/strings/anagram.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/cloudtools/troposphere/tree/master/troposphere/validators.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/extensions/nesting.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/sympy/sympy/tree/master/sympy/crypto/crypto.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/sympy/sympy/tree/master/sympy/crypto/crypto.py":#"https://github.com/yt-dlp/yt-dlp/tree/master/yt_dlp/utils.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/extensions/nesting.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":  # "https://github.com/pymc-devs/pymc/tree/master/pymc/sampling.py":#"https://github.com/smicallef/spiderfoot/tree/master//sflib.py":#"https://github.com/smicallef/spiderfoot/tree/master//sfwebui.py":#
#         #     continue
#         # if file_html!="https://github.com/shuup/shuup/tree/master/shuup_tests/notify/utils.py":#"https://github.com/shuup/shuup/tree/master/shuup/admin/modules/orders/sections.py":#"https://github.com/Diaoul/subliminal/tree/master/subliminal/providers/addic7ed.py":#"https://github.com/capitalone/DataProfiler/tree/master/dataprofiler/profilers/profiler_options.py":#"https://github.com/Axelrod-Python/Axelrod/tree/master/axelrod/classifier.py":#"https://github.com/dpkp/kafka-python/tree/master/kafka/coordinator/assignors/sticky/sticky_assignor.py":#"https://github.com/dpkp/kafka-python/tree/master/kafka/coordinator/assignors/sticky/sticky_assignor.py":#"https://github.com/terraform-compliance/cli/tree/master/terraform_compliance/common/helper.py":#"https://github.com/python-poetry/poetry/tree/master/poetry/puzzle/solver.py":#"https://github.com/Rockyzsu/stock/tree/master//fupan.py":#"https://github.com/OctoPrint/OctoPrint/tree/master/src/octoprint/server/views.py":
#         #     continue
#         print("file_html: ", file_html)
#         try:
#             content = util.load_file_path(file_path)
#         except:
#             print(f"{file_path} is not existed!")
#             continue
#
#         try:
#             file_tree = ast.parse(content)
#             ana_py = ast_util.Fun_Analyzer()
#             ana_py.visit(file_tree)
#             # one_file_chained_comparison_code_list = []
#             #print("func number: ",file_html,len(ana_py.func_def_list))
#             dict_class = dict()
#             for tree, class_name in ana_py.func_def_list:
#                 # if 1:
#                 #print("come here",ast.unparse(tree))
#                 new_code_list = get_complicated_for_comprehen_code_list(tree,content)
#                 for ind, (for_node, assign_node,remove_ass_flag) in enumerate(new_code_list):
#                     new_code_list[ind].append(
#                         transform_comprehension_dict_compli_to_simple.transform(for_node, assign_node))
#                 # for old_code,old_ass,new_code in new_code_list:
#                 #     print("old_code,old_ass,new_code: ",ast.unparse(old_code),old_ass,ast.unparse(new_code))
#
#                 # one_file_chained_comparison_code_list.extend(get_complicated_for_comprehen_code_list(tree,content))
#                 ast_util.set_dict_class_code_list(tree,dict_class, class_name, new_code_list)
#
#                 # count+=len(one_file_chained_comparison_code_list)
#             dict_file[file_html] = dict_class
#             #print(dict_class)
#             # if one_file_chained_comparison_code_list:
#             #     one_repo_chained_comparison_code_list.append(
#             #         [one_file_chained_comparison_code_list, file_path, file_html])
#                 # print("one_file_truth_value_test_code_list: ",one_repo_chained_comparison_code_list)
#                     # break
#         except SyntaxError:
#
#             print("the file has syntax error")
#             continue
#         except ValueError:
#             traceback.print_exc()
#             print("the file has value error: ", file_html)#content,
#             continue
#         except:
#             traceback.print_exc()
#         # break
#     #print("count: ",count)
#     util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
#     print("save successfully! ", count,save_complicated_code_dir_pkl + repo_name)


if __name__ == '__main__':
    code = '''
# def func_1():
components = {}
for i in range(2):
    print(1)
components['trend'] = get_forecast_component_plotly_props(
    m, fcst, 'trend', uncertainty, plot_cap)
if m.train_holiday_names is not None and 'holidays' in fcst:
    components['holidays'] = get_forecast_component_plotly_props(
        m, fcst, 'holidays', uncertainty)

regressors = {'additive': False, 'multiplicative': False}
for name, props in m.extra_regressors.items():
    regressors[props['mode']] = True
for mode in ['additive', 'multiplicative']:
    if regressors[mode] and 'extra_regressors_{}'.format(mode) in fcst:
        components['extra_regressors_{}'.format(mode)] = get_forecast_component_plotly_props(
            m, fcst, 'extra_regressors_{}'.format(mode))
payload={}
a={}
for field in model.__searchable__:
    for i in range(1):
        payload[field] = getattr(model, field)
'''
    # dict_repo_file_python = util.load_json(util.data_root, "python3_repos_files_info")
    #
    # save_one_repo("Real-Time-Voice-Cloning", code1)

    '''
    # print("code1: ",code1)
    tree = ast.parse(code1)
    #code1 = ast.unparse(tree)
    #print("code1: ", code1)

    code_list=get_complicated_for_comprehen_code_list(tree,code1)
    print("len: ",len(code_list))
    for e in code_list:
        print(">>>>>code1： ", e[0],ast.unparse(e[0]))
        print("block: ", e[-1])
        # for each in  e[-1]:
        #     print(each)

        print("----------END------------")
    '''
    # test_code="a=dict()"
    # test_tree = ast.parse(test_code)
    # print(ast.unparse(test_tree))

    # layer_node_list = []
    # ast_util.extract_ast_cur_layer_node(tree, layer_node_list)
    '''
    get comprehension node
    Block=> Assign;...; For
    For=>”for” ( For| If|If-else|body)
    If=>”if” (body| For| If| If-else)
    If-else=>”if” body “else” (body| If-else)
    body=> (Assign|Call)*;Append*
    '''
    # 对于
    # save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/for_compre_dict/"
    # # save_complicated_code_dir = util.data_root + "complicated_code_dir/for_comrephension_dict_complicated_only_one_stmt/"
    # dict_repo_file_python = util.load_json(util.data_root, "python3_1000repos_files_info")
    #
    # #'''
    # # dict_repo_file_python = util.load_json(util.data_root, "python3_1000repos_files_info")
    # repo_list = []
    # for ind, repo_name in enumerate(dict_repo_file_python):
    #     # if repo_name!="shuup":#"subliminal":#"DataProfiler":#"Axelrod":#"kafka-python":#"cli":#"networkx":#"sacred":#"networkx":#"cloud-custodian":#"poetry":#"OctoPrint":#"keras-bert":#"Legofy":
    #     #     continue
    #     # print("repo_name: ", repo_name)
    #     repo_list.append(repo_name)
    # #save_one_repo(repo_list[0])
    # import time
    #
    # start_time = time.time()
    # '''
    # pool = newPool(nodes=30)
    # pool.map(save_one_repo, repo_list)  # [:3]sample_repo_url ,token_num_list[:1]
    # pool.close()
    # pool.join()
    # '''
    #
    # files_num_list = []
    # star_num_list = []
    # contributor_num_list = []
    # count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    # file_list = set([])
    # repo_code_num = dict()
    # result_compli_for_else_list = []
    # all_count_repo, all_file_count, all_me_count = 0, 0, 0
    # for file_name in os.listdir(save_complicated_code_dir_pkl):
    #     all_count_repo += 1
    #     repo_name = file_name[:-4]
    #     # files_num_list.append(repo_files_info[repo_name])
    #     # star_num_list.append(repo_star_info[repo_name])
    #     # contributor_num_list.append(repo_contributor_info[repo_name])
    #
    #     complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)
    #
    #     repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
    #         complicate_code)
    #     # for code_list, file_path, file_html in complicate_code:
    #     code_count += repo_code_count
    #     file_count += repo_file_count
    #     me_count += repo_me_count
    #     all_file_count += repo_all_file_count
    #     all_me_count += repo_all_me_count
    #     repo_exist = 0
    #     for file_html in complicate_code:
    #         for cl in complicate_code[file_html]:
    #             for me in complicate_code[file_html][cl]:
    #                 if complicate_code[file_html][cl][me]:
    #                     repo_exist = 1
    #                     for code in complicate_code[file_html][cl][me]:
    #                         # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
    #                         # print("code1",code1)
    #                         result_compli_for_else_list.append(
    #                             [repo_name, file_html, cl, me, code[1].lineno,
    #                              ast.unparse(code[1]) + "\n" + ast.unparse(code[0]), ast.unparse(code[-1]), code[2]])
    #
    #                         # result_compli_for_else_list.append(
    #                         #     [repo_name, file_html, cl, me, ast.unparse(code1[0]), ast.unparse(code1[1])])
    #
    #         # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
    #     count_repo += repo_exist
    #
    # # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # # print(a)
    # # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    # print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    # import random
    #
    # random.shuffle(result_compli_for_else_list)
    # util.save_csv(util.data_root + "result_csv/for_comprehen_dict.csv",
    #               result_compli_for_else_list[:400],
    #               ["repo_name", "file_html", "class_name", "me_name", "line_no", "old_code", "new_code",
    #                "remove_ass_flag"])
    # end_time = time.time()
    # print("total time: ", end_time - start_time)
    # 1 156 2943 100 1 2990 40102 salt
    # 791 2010 103165 1291 800 121348 1192868
    # util.save_csv(util.data_root + "complicated_code_dir_pkl/for_compre_dict.csv", result_compli_for_else_list,
    #               ["repo_name", "file_html", "class_name", "me_name", "for_code", "assign_code"])














import sys,ast,os

# sys.path.append("/mnt/zejun/smp/code1/")
abs_path=os.path.abspath(os.path.dirname(__file__))

# abs_path=os.getcwd()
# print("abs_path: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
sys.path.append(pack_path)

import time,complicated_code_util
import util
start_time = time.time()
save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/var_unpack_for_target_complicated/"
save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/var_unpack_call_star_complicated/"
# save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/var_unpack_call_star_complicated_strict/"

save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/for_compre_list/"
# save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/for_compre_set/"
save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/for_compre_dict/"
save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/chain_comparison/"
# save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/truth_value_test_complicated_remove_is_is_not/"
# save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/for_else/"
# save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/multip_assign_complicated/"

dict_repo_file_python = util.load_json(util.data_root, "python3_1000repos_files_info")
print("num of repos: ",len(list(dict_repo_file_python.keys())))
repo_name_list=[]
for repo_name in dict_repo_file_python:
        if os.path.exists(save_complicated_code_dir_pkl + repo_name + ".pkl"):

            continue
        #     return None
        # if repo_name!="spiderfoot":
        #     continue
        repo_name_list.append(repo_name)

print("repo num: ", len(repo_name_list))
files_num_list = []
star_num_list = []
contributor_num_list = []
count_repo, file_count, me_count, code_count = 0, 0, 0, 0
file_list = set([])
repo_code_num = dict()
result_compli_for_else_list = []
all_count_repo, all_file_count, all_me_count = 0, 0, 0
for file_name in os.listdir(save_complicated_code_dir_pkl):
    all_count_repo += 1
    repo_name = file_name[:-4]
    # files_num_list.append(repo_files_info[repo_name])
    # star_num_list.append(repo_star_info[repo_name])
    # contributor_num_list.append(repo_contributor_info[repo_name])

    complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)

    repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
        complicate_code)
    # for code_list, file_path, file_html in complicate_code:
    code_count += repo_code_count
    file_count += repo_file_count
    me_count += repo_me_count
    all_file_count += repo_all_file_count
    all_me_count += repo_all_me_count
    repo_exist = 0
    for file_html in complicate_code:
        for cl in complicate_code[file_html]:
            for me in complicate_code[file_html][cl]:
                if complicate_code[file_html][cl][me]:
                    repo_exist = 1
                    for code in complicate_code[file_html][cl][me]:
                        pass
                        # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
                        #                code_index_start_end_list.append([node,assign_stmt,node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])

                        # result_compli_for_else_list.append(
                        #     [repo_name, file_html, cl, me, ast.unparse(code1[0]), ast.unparse(code1[1])])

        # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
    count_repo += repo_exist

# a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
# print(a)
# print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
# print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
# print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
# print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
end_time = time.time()
print("total time: ", end_time - start_time)


import sys,ast,os,copy
import tokenize
# sys.path.append("/mnt/zejun/smp/code1/")
abs_path=os.path.abspath(os.path.dirname(__file__))

# abs_path=os.getcwd()
# print("abs_path: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
# print(pack_path)
sys.path.append(pack_path)
import util
# from pathos.multiprocessing import ProcessingPool as newPool

'''
input: 代码片段 code1
if if,while,assert节点
    if test是compare节点
        
elif if boolOP节点
    if values 中含有compare 节点


'''
def decide_compare_complicate_truth_value(test):
    flag=0
    ops = test.ops
    left = ast.unparse(test.left)
    comparators = test.comparators
    comp_str = ast.unparse(comparators[0])

    empty_list = ["None", "True", "False", "0", "0.0", "0j", "Decimal(0)", "Fraction(0, 1)", '', "()","[]", "{}", "dict()", "set()", "range(0)"]

    if len(ops) == 1 and isinstance(ops[0], (ast.Eq, ast.NotEq, ast.Is, ast.IsNot)):
        if left in empty_list or comp_str in empty_list:
            flag=1
    return flag
def get_truth_value_test(code):
    code_list = []
    tree = ast.parse(code)
    for node in ast.walk(tree):

        if isinstance(node, (ast.If, ast.While, ast.Assert)):
            test = node.test
            if isinstance(test, ast.Compare):
                if decide_compare_complicate_truth_value(test):
                    code_list.append(ast.unparse(test))


        elif isinstance(node, ast.BoolOp):
            for value in node.values:
                if isinstance(value, ast.Compare):
                    if decide_compare_complicate_truth_value(value):
                        code_list.append(ast.unparse(value))

    # for e in code_list:
    #     print("code1: ", e)
    return code_list
def save_repo_for_else_complicated(repo_name):
    # for ind,repo_name in enumerate(dict_repo_file_python):
        # if "spotify" not in repo_name:
        #     continue
        # print("come here")
        one_repo_chained_comparison_code_list = []
        for file_info in dict_repo_file_python[repo_name]:
            file_path = file_info["file_path"]
            file_html = file_info["file_html"]
            try:
                content = util.load_file(file_path)
            except:
                print(f"{file_path} is not existed!")
                continue

            try:

                one_file_chained_comparison_code_list=get_truth_value_test(content)
                if one_file_chained_comparison_code_list:
                    one_repo_chained_comparison_code_list.append([one_file_chained_comparison_code_list, file_path, file_html])
                    # print("one_file_truth_value_test_code_list: ",one_file_truth_value_test_code_list)
                    # break
            except SyntaxError:
                print("the file has syntax error")
                continue
            except ValueError:
                print("the file content has some error",file_html)
                continue
            # break
        if one_repo_chained_comparison_code_list:
            # count_repo+=1
            # print("it exists truth value test complicated code1: ", len(one_repo_for_else_code_list))
            util.save_json(save_complicated_code_dir, repo_name, one_repo_chained_comparison_code_list)
if __name__ == '__main__':
    code='''
if self . get_conf_value ( 'show' , header = header ) == [ ] :
    pass
elif stats_grab == { } :
    pass
assert policy . remember ( pretend . stub ( ) , pretend . stub ( ) ) == 0
while a!=[]:
    pass

'''



    # ana_py = Analyzer()
    # ana_py.visit(file_tree)
    # print("fun number: ",len(ana_py.func_def_list))

    save_complicated_code_dir= util.data_root + "complicated_code_dir/truth_value_test_complicated/"

    #dict_repo_file_python=util.load_json(util.data_root, "python3_repos_files_info" )
    # dict_repo_file_python=util.load_json(util.data_root, "python3_1000repos_files_info_modify" )
    dict_repo_file_python=util.load_json(util.data_root, "python3_1000repos_files_info" )


    repo_name_list=[]
    for repo_name in dict_repo_file_python:

        repo_name_list.append(repo_name)
    '''
    count_complicated_code=0
    for ind,repo_name in enumerate(dict_repo_file_python):
        one_repo_truth_value_test_code_list = []
        for file_info in dict_repo_file_python[repo_name]:
            file_path = file_info["file_path"]
            file_html = file_info["file_html"]
            content = util.load_file(file_path)
            try:
            #if 1:
                one_file_truth_value_test_code_list=get_truth_value_test(content)
                count_complicated_code+=len(one_file_truth_value_test_code_list)
                if one_file_truth_value_test_code_list:
                    one_repo_truth_value_test_code_list.append([one_file_truth_value_test_code_list, file_path, file_html])
                    # print("one_file_truth_value_test_code_list: ",one_file_truth_value_test_code_list)
                    # break
            except SyntaxError:
                print("the file has syntax error")
                continue
            # break
        if one_repo_truth_value_test_code_list:
            # print("it exists truth value test complicated code1: ", len(one_repo_for_else_code_list))
            util.save_json(save_complicated_code_dir, repo_name, one_repo_truth_value_test_code_list)

        # break
    # print()


        # break
    print("count_complicated_code: ",ind,count_complicated_code)
    
    '''
    '''
    pool = newPool(nodes=30)
    pool.map(save_repo_for_else_complicated, repo_name_list)  # [:3]sample_repo_url ,token_num_list[:1]
    pool.close()
    pool.join()
    # print("len all_files: ", len(all_files))
    '''
    #'''
    count=0
    result_compli_for_else_list=[]
    for file_name in os.listdir(save_complicated_code_dir):
        complicate_code=util.load_json(save_complicated_code_dir,file_name[:-5])
        for code_list, file_path,file_html in complicate_code:
            count += len(code_list)
            for code in code_list:
                repo_name=file_html.split("/")[4]
                result_compli_for_else_list.append([repo_name,code,file_html,file_path])
            #     print("one code1: ",repo_name,code1,file_html,file_path)
            #     break
            # break
        # print("file: ",file_name)
        # break
    print("count: ",count,len(os.listdir(save_complicated_code_dir)))
    # util.save_csv(util.data_root+"complicated_code_dir/truth_value_test_complicated.csv",result_compli_for_else_list,["repo_name","code1","file_html","file_path"])
    #'''











    # print("----------------------------\n")
    # for code1 in for_else_filter_redundant_code_list:
    #     print("each code1: ", code1[0])



import sys,ast, copy,os


abs_path=os.path.abspath(os.path.dirname(__file__))
# print("abs_path: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
# print(pack_path)
sys.path.append(pack_path)
import util


def copy_compre(node):
    compr = ast.comprehension()
    compr.target = node.target
    compr.iter = node.iter
    compr.ifs=[]
    compr.is_async=None
    # print("compr: ",ast.unparse(compr))
    return compr


def for_transform(node,listcompre):
    # print("node: ",node,ast.unparse(node))

    if isinstance(node,ast.For):
        body = node.body[0]
        listcompre.generators.append(copy_compre(node))

        for_transform(body,listcompre)
        # print("come to the ast.For hahahha: ",ast.unparse(listcompre))

    elif isinstance(node,ast.If):
        body = node.body[0]
        if not node.orelse:
            listcompre.generators[-1].ifs.append(node.test)
            for_transform(body, listcompre)
            pass
        else:
            def if_else_transform(node):
                if isinstance(node,ast.If):
                    ifexpr = ast.IfExp()
                    ifexpr.test = node.test

                    ifexpr.body = node.body[0].value.args[0]
                    ifexpr.orelse = if_else_transform(node.orelse[0])
                    return ifexpr
                else:
                    return node.value.args[0]

            listcompre.elt =if_else_transform(node)
            # ifexpr=ast.IfExp()
            # ifexpr.test=node.test
            #
            # ifexpr.body=body.value.args[0]
            # ifexpr.orelse=None
            # for_transform(node.orelse[0], ifexpr.orelse)
            #
            # # ifexpr.orelse=listcompre.elt
            # listcompre.elt = ifexpr

    else:

        listcompre.elt = node.value.args[0]


def test_fun(a):
    a.id="change"
    print()
def transform(for_node, assign_node):
    new_code=copy.deepcopy(assign_node)

    listcompre=ast.SetComp()
    listcompre.generators=[]
    listcompre.elt=None
    for_transform(for_node,listcompre)
    new_code.value = listcompre
    return new_code
def replace_file_content(for_node,assign_stmt,new_code,real_file_html,repo_name):
    rela_path = "/".join(real_file_html.split("/")[6:])
    file_path = "".join([util.data_root, "python_star_2000repo/", repo_name, "/", rela_path])
    content = util.load_file_path(file_path)
    res_copy = content.split("\n")
    indent = ""
    for ind, e in enumerate(res_copy[for_node.lineno - 1]):
        if e != " ":
            indent = " " * ind
            break

    res_copy[assign_stmt.lineno - 1] = indent + ast.unparse(new_code)
    res_copy[for_node.lineno - 2:for_node.end_lineno] = res_copy[for_node.lineno - 2:for_node.lineno - 1]
    return content,"\n".join(res_copy)
def traverse(node, complicate_code, assign_str):
    if isinstance(node, ast.For):
        target = node.target
        iter = node.iter
        body = node.body[0]
        complicate_code.append("for " + ast.unparse(target) + " in " + ast.unparse(iter) + " ")
        traverse(body, complicate_code, assign_str)



    elif isinstance(node, ast.If):
        test = node.test
        body = node.body[0]
        if not node.orelse:

            complicate_code.append("if " + ast.unparse(test) + " ")
            traverse(body, complicate_code, assign_str)
        else:  # 这个是一个赋值语句
            if isinstance(body, ast.Expr) and isinstance(body.value, ast.Call):
                args_if = ast.unparse(body.value.args)

            or_else = node.orelse[0]
            if isinstance(or_else, ast.Expr) and isinstance(or_else.value, ast.Call):
                args_or_else = ast.unparse(or_else.value.args)

                assign_str.append(args_if + " if " + ast.unparse(test) + " else " + args_or_else + " ")
            else:
                assign_str.append(args_if + " if " + ast.unparse(test) + " else ")
                traverse(or_else, complicate_code, assign_str)
            # print(args_or_else)
            # complicate_code.insert(0,args_if+" if " + ast.unparse(test) + " else "+args_or_else+" ")

    elif isinstance(node, ast.Expr) and isinstance(node.value, ast.Call):
        assign_str.append(ast.unparse(node.value.args) + " ")
        # complicate_code.insert(0,args_assign_expr)
if __name__ == '__main__':
    code='''
#a=[i if i>2 else 0 for i in range(10)]
#a.b=[i if i>2 else 0 for i in range(10) if i>2 if i>6 if i+7>9 for j in range(3)]
#matches = []
# for (root, dirnames, filenames) in os.walk(stem):
#     for filename in fnmatch.filter(filenames, file_pattern):
#         matches.append(path_join_robust(root, filename))
    
# for f in files:
#     if not stem or f.startswith(stem + '/'):
#         matches.append(('', '_', [f]))
        
# for (xidx, yy) in enumerate(edge_ccs):
#     if invalid is not True and len(context_ccs[xidx]) > 0:
#         valid_edge_ccs.append(yy)
#     elif invalid is True and len(context_ccs[xidx]) == 0:
#         valid_edge_ccs.append(yy)
#     else:
#         valid_edge_ccs.append(set())

# closest_depths=[]
# for (closest_px, closest_py) in zip(closest_pxs, closest_pys):
#     if info_on_pix.get((closest_px + start_near_node[0] - 1 + anchor[0], closest_py + start_near_node[1] - 1 + anchor[2])) is not None:
#         for info in info_on_pix.get((closest_px + start_near_node[0] - 1 + anchor[0], closest_py + start_near_node[1] - 1 + anchor[2])):
#             if info['synthesis'] is False:
#                 closest_depths.append(abs(info['depth']))

dataset_ids=set([])
for d in params.get('datasets'):
    if isinstance(d, list):
        dataset_ids.append(d[0])
    else:
        dataset_ids.append(d)
# sparse_vectors_sequence=[]
# for token in sequence:
#     sparse_vectors_sequence.append(voc.binary_vocabulary[token])
            
# a=[]
# for i in range(3):
#     if i>func(i):
#         for j in range(6):
#             if j > i+6:
#                 if i>3:
#                     a.append(i+3)
#                 else:
#                     if i>4:
#                         a.append(i+3)
#                     else:
#                         a.append(i+1)
'''
    tree= ast.parse(code)
    complicate_code=[]
    transform_node=[]
    Var=""
    '''
    complicate_code=[]
    assign_left_code=[]
    for node in ast.walk(tree):
        if isinstance(node,ast.Assign):
            left=node.targets
            Var=ast.unparse(left)
            assign_left_code.append(Var+" = [ ")


        elif isinstance(node,ast.For):
            assign_str=[]
            traverse(node, complicate_code,assign_str)
            complicate_code.insert(0,"".join(assign_str))
            complicate_code.append("]")
            break
    '''
    for_node=None
    ass_node=None
    for node in ast.walk(tree):
        if isinstance(node, ast.Assign):
            ass_node=node
        elif isinstance(node,ast.For):
            for_node=node

    new_code=transform(for_node,ass_node)
    print("new_code: ",ast.unparse(new_code))
    '''
    assign_left_code.extend(complicate_code)
    str_complicate_code="".join(assign_left_code)
    print(str_complicate_code)
    '''

















import sys,ast,os
abs_path=os.path.abspath(os.path.dirname(__file__))

pack_path="/".join(abs_path.split("/")[:-1])
sys.path.append(pack_path)
# sys.path.append("/mnt/zejun/smp/code1/")
import copy, time,complicated_code_util,util

from RefactoringIdioms.extract_simp_cmpl_data import ast_util

from RefactoringIdioms.transform_c_s import transform_var_unpack_for_target_compli_to_simple_new
from pathos.multiprocessing import ProcessingPool as newPool
import traceback
'''
1. for.body 中不包含对target的写操作，即在assign的左边
2. 对于for.body 中只包含对target的读操作，subscript，并且slice是常量
'''
def whether_slice_is_constant(slice,var):
    # print("slice: ",ast.unparse(slice),slice)
    if isinstance(slice, ast.Constant):
        slice_value = slice.value
        if isinstance(slice_value, int):
           return True
        else:
            return False
    elif isinstance(slice, ast.Slice):
        # print("slice: ", slice.__dict__, ast.unparse(slice))
        lower = -1
        upper = -1
        step = -1
        if slice.lower:
            if isinstance(slice.lower, ast.Constant):
                lower = ast.unparse(slice.lower)
            else:
                return False
        else:
            lower = 0
        if slice.upper:
            if isinstance(slice.upper, ast.Constant):
                upper = ast.unparse(slice.upper)
            else:
                return False
        else:
            upper = ''.join(['len(', var, ')'])
        if slice.step:
            if isinstance(slice.step, ast.Constant):
                if isinstance(slice.step.value, int) and slice.step.value == 1:
                    step = 1
            else:
                return False
        else:
            step = 1
    return False

def is_var_subscript(node,var):

    if isinstance(node, ast.Subscript):
        value = node.value

        if ast.unparse(value) == var:
            # return True
            slice = node.slice
            return whether_slice_is_constant(slice,var)


        elif  isinstance(value, ast.Subscript):
            return is_var_subscript(value,var)
        else:
            return False
    return False
def is_var(node,var_ast):
    if isinstance(node, type(var_ast)) and ast.unparse(node) == ast.unparse(var_ast):
        return True
    return False
'''
判断给定节点是否只包含var的subscript使用，并且subscript的slice都是常量
'''
def whether_var_subscript(node,target,var_list=[]):
    # print("node: ", ast.unparse(node))
    var=ast.unparse(target)
    if is_var_subscript(node,var):
        # var_list.add(ast.unparse(node))
        var_list.add(node)
        return True
        pass
    elif is_var(node,target):
        return False
        pass
    else:

        for e in ast.iter_child_nodes(node):
            if not whether_var_subscript(e,target,var_list):
                return False

                # var_list.add(ast.unparse(e))
        return True
def get_for_target(tree):
    code_list=[]

    for node in ast.walk(tree):
        if isinstance(node, ast.For):
            var_list = set([])
            target = node.target
            var = ast.unparse(target)
            # print("var: ",var)
            # print("target: ",target.__dict__, type(target))
            count_obj = ast_util.get_basic_count(target)
            if count_obj == 1:

                for e_body in ast.walk(node):
                    # if isinstance(e_body, (ast.Assign)):
                    if isinstance(e_body, (ast.Assign,ast.AnnAssign,ast.AugAssign)):
                        if hasattr(e_body,'targets'):
                            left = e_body.targets
                        elif hasattr(e_body,'target'):
                            left = [e_body.target]

                        for left_e in left:
                            for ass_e in ast.walk(left_e):
                                # print("the ele: ",ast.unparse(left_e))
                                if ast.unparse(ass_e) == var:
                                    # print("it cannot be simplified because it contains write operation for the var ",var)
                                    break
                            else:
                                continue
                            break
                        else:
                            continue
                        break
                else:
                    '''
                    whether code1  use target
                    '''
                    flag=0
                    for stmt in node.body:
                        for e in ast.walk(stmt):
                            if is_var(e,target):
                                # print("come here: ",ast.unparse(stmt))
                                flag=1
                                break
                    else:
                        pass
                    if flag:
                        for stmt in node.body:
                            '''
                            whether code1  use target without subscript, if use it cannot simplify
                            '''
                            flag=whether_var_subscript(stmt, target,var_list)
                            if not flag:
                                # print("it cannot be simplified because it contains other operation for the var besides the subscript operation ", var,ast.unparse(stmt),flag)

                                break
                        else:
                            var_list_real = list(var_list)
                            #print("var_list_real: ",var_list_real)#['val[1]', 'val[1][0]', 'val[1][1]', 'val[0]']-->['val[1]', 'val[0]']
                            for i,var1 in enumerate(var_list):
                                for j,var2 in enumerate(var_list):
                                    if ast.unparse(var1) in ast.unparse(var2) and ast.unparse(var1)!=ast.unparse(var2):
                                        var_list_real.remove(var2)
                            # print("var_list_real: ",var_list_real)
                            Map_var=dict()
                            node_copy=copy.deepcopy(node)
                            # print(">>>>>>>>>>>node_copy: ", ast.unparse(node_copy),var_list_real)
                            # for var in var_list_real:
                            #     print("var: ",ast.unparse(var))
                            new_code= transform_var_unpack_for_target_compli_to_simple_new.transform_for_node_var_unpack(node_copy, var_list_real, Map_var)
                            # print("Map_Var: ",Map_var)
                            if new_code:
                                # print("new_code: ",ast.unparse(new_code))
                                code_list.append([node,new_code])
                            else:
                                code_list.append([node])
                                # print(">>>>>>>>>>>the code1 I cannot transform: ",ast.unparse(node))
                        # print("it maybe simplified")
    return code_list
def get_for_target_add_info(tree):
    code_list=[]

    for node in ast.walk(tree):
        if isinstance(node, ast.For):
            var_list = set([])
            target = node.target
            var = ast.unparse(target)
            # print("var: ",var)
            # print("target: ",target.__dict__, type(target))
            count_obj = ast_util.get_basic_count(target)
            if count_obj == 1:

                for e_body in ast.walk(node):
                    # if isinstance(e_body, (ast.Assign)):
                    if isinstance(e_body, (ast.Assign,ast.AnnAssign,ast.AugAssign)):
                        if hasattr(e_body,'targets'):
                            left = e_body.targets
                        elif hasattr(e_body,'target'):
                            left = [e_body.target]

                        for left_e in left:
                            for ass_e in ast.walk(left_e):
                                # print("the ele: ",ast.unparse(left_e))
                                if ast.unparse(ass_e) == var:
                                    # print("it cannot be simplified because it contains write operation for the var ",var)
                                    break
                            else:
                                continue
                            break
                        else:
                            continue
                        break
                else:
                    '''
                    whether code1  use target
                    '''
                    flag=0
                    for stmt in node.body:
                        for e in ast.walk(stmt):
                            if is_var(e,target):
                                # print("come here: ",ast.unparse(stmt))
                                flag=1
                                break
                    else:
                        pass
                    if flag:
                        for stmt in node.body:
                            '''
                            whether code1  use target without subscript, if use it cannot simplify
                            '''
                            flag=whether_var_subscript(stmt, target,var_list)
                            if not flag:
                                # print("it cannot be simplified because it contains other operation for the var besides the subscript operation ", var,ast.unparse(stmt),flag)

                                break
                        else:
                            var_list_real = list(var_list)
                            #print("var_list_real: ",var_list_real)#['val[1]', 'val[1][0]', 'val[1][1]', 'val[0]']-->['val[1]', 'val[0]']
                            for i,var1 in enumerate(var_list):
                                for j,var2 in enumerate(var_list):
                                    if ast.unparse(var1) in ast.unparse(var2) and ast.unparse(var1)!=ast.unparse(var2):
                                        var_list_real.remove(var2)
                            # print("var_list_real: ",var_list_real)
                            Map_var=dict()
                            node_copy=copy.deepcopy(node)
                            # print(">>>>>>>>>>>node_copy: ", ast.unparse(node_copy),var_list_real)
                            # for var in var_list_real:
                            #     print("var: ",ast.unparse(var))
                            new_code= transform_var_unpack_for_target_compli_to_simple_new.transform_for_node_var_unpack(node_copy, var_list_real, Map_var)
                            # print("Map_Var: ",Map_var)
                            if new_code:
                                # print("new_code: ",ast.unparse(new_code))
                                code_list.append([node,new_code,var_list_real,Map_var])
                            else:
                                code_list.append([node,var_list_real,Map_var])
                                # print(">>>>>>>>>>>the code1 I cannot transform: ",ast.unparse(node))
                        # print("it maybe simplified")
    return code_list
def transform_for_multiple_targets_code(content):
    code_pair_list = []
    # print(content)
    try:

        file_tree = ast.parse(content)
        ana_py = ast_util.Fun_Analyzer()
        ana_py.visit(file_tree)

        for tree, class_name in ana_py.func_def_list:
            try:
                me_name = tree.name
            except:
                me_name = ""
            new_code_list = get_for_target(tree)

            # print("new_code_list: ",new_code_list)
            for e in new_code_list:
                # print("new_tree: ",ast.unparse(e[0]),ast.unparse(e[1]))
                if len(e) < 2:
                    print(">>>>>>>>>>>the code1 I cannot transform: ", e, ast.unparse(e))
                    continue

                # new_file_content = replace_file_content_ass(content, each_assign_list, new_code)
                # assign_list[ind_ass].append(new_code)
                # complic_code_me_info_dir_pkl = util.data_root + "complic_code_me_info_dir_pkl/each_idiom_type_all_methods/multi_ass/"  # for_else
                # util.save_file(complic_code_me_info_dir_pkl, "test"+str(ind_ass), new_file_content, ".txt", "w")
                # util.save_file(complic_code_me_info_dir_pkl, "test_old"+str(ind_ass), content, ".txt", "w")
                old_node_content = ast.get_source_segment(content, e[0])

                line_list=[[(e[0].lineno,e[0].col_offset),(e[0].end_lineno,e[0].end_col_offset)]]

                new_code_fragment_list=ast.unparse(e[1]).split('\n')

                for ind,e_fragment in enumerate(new_code_fragment_list):

                    if ind>0:

                        new_code_fragment_list[ind]=' '*e[0].col_offset+e_fragment

                print("for multi targets ast.unparse(e[1]): ",ast.unparse(e[1]))
                # code_pair_list.append([class_name,me_name,[old_node_content], [ast.unparse(e[1])],line_list])
                code_pair_list.append([class_name,me_name,[old_node_content], ["\n".join(new_code_fragment_list)],line_list])

        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list

def get_for_multi_tar_part_fragment(content):
    code_pair_list = []

    try:

            tree = ast.parse(content)

            try:
                me_name = tree.name
            except:
                me_name = ""
            new_code_list = get_for_target(tree)

            # print("new_code_list: ",new_code_list)
            for e in new_code_list:
                # print("new_tree: ",ast.unparse(e[0]),ast.unparse(e[1]))
                if len(e) < 2:
                    print(">>>>>>>>>>>the code1 I cannot transform: ", e, ast.unparse(e))
                    continue

                # new_file_content = replace_file_content_ass(content, each_assign_list, new_code)
                # assign_list[ind_ass].append(new_code)
                # complic_code_me_info_dir_pkl = util.data_root + "complic_code_me_info_dir_pkl/each_idiom_type_all_methods/multi_ass/"  # for_else
                # util.save_file(complic_code_me_info_dir_pkl, "test"+str(ind_ass), new_file_content, ".txt", "w")
                # util.save_file(complic_code_me_info_dir_pkl, "test_old"+str(ind_ass), content, ".txt", "w")
                if 1:#target_node_str == ast.unparse(e[0]):
                    print("come for multi targets")
                    old_node_content = ast.get_source_segment(content, e[0])
                    line_list=[[(e[0].lineno,e[0].col_offset),(e[0].end_lineno,e[0].end_col_offset)]]

                    new_code_fragment_list=ast.unparse(e[1]).split('\n')

                    for ind,e_fragment in enumerate(new_code_fragment_list):

                        if ind>0:

                            new_code_fragment_list[ind]=' '*e[0].col_offset+e_fragment


                    # code_pair_list.append([class_name,me_name,[old_node_content], [ast.unparse(e[1])],line_list])
                    code_pair_list.append(['',me_name,[old_node_content], ["\n".join(new_code_fragment_list)],line_list])

            return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list
def save_repo_for_else_complicated(repo_name):
    start=time.time()
    count_complicated_code = 0
    print("come the repo: ", repo_name)
    one_repo_for_else_code_list = []

    dict_file=dict()
    # if os.path.exists(save_complicated_code_dir_pkl + repo_name + ".pkl"):
    #     print("the repo has been saved before")
    #     return None

    for ind,file_info in enumerate(dict_repo_file_python[repo_name]):

        file_path = file_info["file_path"]
        # if file_path!="/mnt/zejun/smp/data/python_repo_1000/VideoPose3D//run.py":
        #     continue
        file_html = file_info["file_html"]
        # print(file_html)
        # if file_html!="https://github.com/nipy/nibabel/tree/master/nibabel/fileslice.py":  #
        #     continue
        # if file_html!="https://github.com/raphaelvallat/pingouin/tree/master/pingouin/regression.py":#"https://github.com/smicallef/spiderfoot/tree/master//sflib.py":#"https://github.com/smicallef/spiderfoot/tree/master//sfwebui.py":#"https://github.com/PaddlePaddle/Paddle/tree/master/python/paddle/dataset/tests/imikolov_test.py":
        #     continue
        # print("cone here")
        try:
            content = util.load_file_path(file_path)
        except:
            print(f"{file_path} is not existed!")
            continue

        # print("content: ",content)
        try:
            # print("come here")
            file_tree = ast.parse(content)
            ana_py = ast_util.Fun_Analyzer()
            ana_py.visit(file_tree)
            # print("ana_py.func_def_list ", ana_py.func_def_list)
            # dict_file["repo_name"]=repo_name
            dict_class=dict()
            for tree, class_name in ana_py.func_def_list:

                new_code_list=get_for_target(tree)
                new_code_list_new=[]
                # print("new_code_list: ",new_code_list)
                for e in new_code_list:
                    # print("new_tree: ",ast.unparse(e[0]),ast.unparse(e[1]))
                    if len(e)<2:
                        print(">>>>>>>>>>>the code1 I cannot transform: ", file_html,e,ast.unparse(e))
                        continue
                    new_code_list_new.append(e)
                # print("new_code_list ", new_code_list)
                #
                # for old_code,new_code in new_code_list:
                #     print("old_code,new_code: ",ast.unparse(old_code),ast.unparse(new_code))
                ast_util.set_dict_class_code_list(tree,dict_class, class_name, new_code_list_new)
            dict_file[file_html]=dict_class
        except SyntaxError:
            print("the file has syntax error")
            continue
        except ValueError:
            print("the file has value error: ", content, file_html)
            continue
        # break
    end = time.time()
    #'''
    if dict_file:
        count_complicated_code = count_complicated_code + len(one_repo_for_else_code_list)
        # print("it exists for else complicated code1: ", len(one_repo_for_else_code_list))
        util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
        # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
        print(end-start," save successfully! ", save_complicated_code_dir_pkl + repo_name)
    else:
        print(end-start," the repo has no for else")
        util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
        # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
    #'''
    return count_complicated_code
if __name__ == '__main__':
    code='''
for T in a:
    c=1
    d=c+1
    e=T[0]+T[1]
    f=funct(T[1],T[:3])
    if len(T)>1:
        print("nothing")
# for ne_node in ne_nodes:
#     store_tuple = [ne_node, ne_node[i], mesh_nodes[ne_node]['cur_id']]
#     ne_node[0][0]
#     ne_node[0][1]
#     if ne_node[0] == node[0]:
#         if ne_node[1] == ne_node[1] - 1:
#             four_dir_nes['left'].append(store_tuple)
# for _ in range(i):
#     result.append(s)            
# for ne_node in ne_nodes:
#     store_tuple = [ne_node[0], mesh_nodes['cur_id']]
# 
#     if ne_node[0] == node[0]:
#         if ne_node[1] == ne_node[1] - 1:
#             four_dir_nes['left'].append(store_tuple)
# for T  in model._meta.backrefs.items():
# 
#     if T[0].backref == '+':
#         continue
for val in sorted(npz.items()):
    logging.info('  Loading %s in %s' % (str(val[1][0].shape), val[0]))
    logging.info('  Loading %s in %s' % (str(val[1][1].shape), val[0]))
    weights.extend(val[1])
    if len(model.all_weights) == len(weights):
        break
'''
    # tree=ast.parse(code1)
    # code_list=get_for_target(tree)
    # for code1,new_code in code_list:
    #     print("code1:",ast.unparse(code1))
    #     print("new_code",ast.unparse(new_code))

    save_complicated_code_dir = util.data_root + "complicated_code_dir/var_unpack_for_target_complicated/"
    save_complicated_code_dir_pkl= util.data_root + "transform_complicate_to_simple_pkl/var_unpack_for_target_complicated/"
    # dict_repo_file_python = util.load_json(util.data_root, "python3_repos_files_info")
    dict_repo_file_python= util.load_json(util.data_root, "python3_1000repos_files_info")
    repo_name_list=[]
    for repo_name in dict_repo_file_python:
        # if os.path.exists(save_complicated_code_dir_pkl + repo_name + ".pkl"):
        #     continue
        #     return None
        # if repo_name!="nibabel":#"pingouin":#"spiderfoot":
        #     continue
        repo_name_list.append(repo_name)

    print("repo num: ", len(list(dict_repo_file_python.keys())),len(repo_name_list))
    # save_repo_for_else_complicated(repo_name_list[0])
    '''
    count_complicated_code = 0
    count_repo = 0
    for ind, repo_name in enumerate(dict_repo_file_python):
        one_repo_chained_comparison_code_list = []
        for file_info in dict_repo_file_python[repo_name]:
            file_path = file_info["file_path"]
            file_html = file_info["file_html"]
            content = util.load_file(file_path)
            try:
                # if 1:
                one_file_chained_comparison_code_list = get_for_target(content)
                count_complicated_code += len(one_file_chained_comparison_code_list)
                if one_file_chained_comparison_code_list:
                    one_repo_chained_comparison_code_list.append(
                        [one_file_chained_comparison_code_list, file_path, file_html])
                    # print("one_file_truth_value_test_code_list: ",one_file_truth_value_test_code_list)
                    # break
            except SyntaxError:
                print("the file has syntax error")
                continue
            # break
        if one_repo_chained_comparison_code_list:
            # print(one_repo_chained_comparison_code_list)
            # break
            count_repo += 1
            # print("it exists truth value test complicated code1: ", len(one_repo_for_else_code_list))
            util.save_json(save_complicated_code_dir, repo_name, one_repo_chained_comparison_code_list)
                    # for righ_e in ast.walk(right):
                    #     if isinstance(node, ast.Subscript):
                    #         value = node.value
                    #         slice = node.slice
                    #         if ast.unparse(value) == var:
                    #             if isinstance(slice, ast.Constant):
                    #                 slice_value = slice.value
                    #                 if isinstance(slice_value, int):
    
    print("count_complicated_code: ",ind,count_complicated_code)
    '''

    start_time=time.time()
    '''
    pool = newPool(nodes=30)
    pool.map(save_repo_for_else_complicated, repo_name_list)  # [:3]sample_repo_url ,token_num_list[:1]
    pool.close()
    pool.join()
    end_time = time.time()
    print("total time: ", end_time - start_time)
    '''
    files_num_list = []
    star_num_list = []
    contributor_num_list = []
    count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    file_list = set([])
    repo_code_num = dict()
    result_compli_for_else_list = []
    all_count_repo, all_file_count, all_me_count = 0, 0, 0
    for file_name in os.listdir(save_complicated_code_dir_pkl):
        all_count_repo += 1
        repo_name = file_name[:-4]
        # files_num_list.append(repo_files_info[repo_name])
        # star_num_list.append(repo_star_info[repo_name])
        # contributor_num_list.append(repo_contributor_info[repo_name])

        complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)

        repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
            complicate_code)
        # for code_list, file_path, file_html in complicate_code:
        code_count += repo_code_count
        file_count += repo_file_count
        me_count += repo_me_count
        all_file_count += repo_all_file_count
        all_me_count += repo_all_me_count
        repo_exist = 0
        for file_html in complicate_code:
            for cl in complicate_code[file_html]:
                for me in complicate_code[file_html][cl]:
                    if complicate_code[file_html][cl][me]:
                        repo_exist = 1
                        for code in complicate_code[file_html][cl][me]:
                            # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
                            #                code_index_start_end_list.append([node,assign_stmt,node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])

                            result_compli_for_else_list.append(
                                [repo_name, file_html, cl, me, code[0].lineno,ast.unparse(code[0]), ast.unparse(code[1])])

            # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
        count_repo += repo_exist

    # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # print(a)
    # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    import random

    random.shuffle(result_compli_for_else_list)
    util.save_csv(util.data_root + "result_csv/var_unpack_for_target_complicated.csv", result_compli_for_else_list[:400],
                  ["repo_name", "file_html", "class_name", "me_name", "line_no", "old_code", "new_code"])
    end_time = time.time()
    print("total time: ",end_time-start_time)
    # 1 156 2943 100 1 2990 40102 salt
    # 791 2010 103165 1291 800 121348 1192868
    # util.save_csv(util.data_root + "transform_complicate_to_simple_pkl/var_unpack_for_target_complicated.csv", result_compli_for_else_list,
    #               ["repo_name", "file_html", "class_name", "me_name", "for_code", "assign_code"])

    # count = 0
    # result_compli_for_else_list = []
    # for file_name in os.listdir(save_complicated_code_dir):
    #     complicate_code = util.load_json(save_complicated_code_dir, file_name[:-5])
    #     for each_file in complicate_code:
    #         code_list = each_file[0]
    #         file_path = each_file[1]
    #         file_html = each_file[2]
    #         count += len(code_list)
    #         # print("count: ", count)
    #         for code1 in code_list:
    #             repo_name = file_html.split("/")[4]
    #             result_compli_for_else_list.append([repo_name, code1[0],code1[1], file_html, file_path])
    #     #     print("one code1: ",repo_name,code1,file_html,file_path)
    #     #     break
    #     # break
    #     # print("file: ",file_name)
    #     # break
    # print("count: ", count, len(os.listdir(save_complicated_code_dir)))
    # util.save_csv(util.data_root + "complicated_code_dir/var_unpack_for_target_complicated.csv",
    #               result_compli_for_else_list, ["repo_name", "code1","var_list", "file_html", "file_path"])
 #   '''









import sys,ast

# sys.path.append("/Users/zhangzejunzhangzejun/PycharmProjects/pythonProject/python-ast-explorer-master/extract_transform_complicate_code_new/")
# from pathos.multiprocessing import ProcessingPool as newPool
from RefactoringIdioms.transform_c_s import transform_truth_value_test_compli_to_simple
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
# from pathos.multiprocessing import ProcessingPool as newPool
import traceback
'''
input: 代码片段 code1
if if,while,assert节点
    if test是compare节点
        
elif if boolOP节点
    if values 中含有compare 节点


'''
def decide_compare_complicate_truth_value(test):
    flag=0
    ops = test.ops
    left = ast.unparse(test.left)
    comparators = test.comparators
    comp_str = ast.unparse(comparators[0])

    empty_list = ["None", "True", "False", "0", "0.0", "0j", "Decimal(0)", "Fraction(0, 1)", '', "()","[]", "{}", "dict()", "set()", "range(0)"]

    if len(ops) == 1 and isinstance(ops[0], (ast.Eq, ast.NotEq)):
        if left in empty_list or comp_str in empty_list:
            flag=1
    return flag
def get_truth_value_test(tree):
    code_list = []

    for node in ast.walk(tree):

        if isinstance(node, (ast.If, ast.While, ast.Assert)):
            test = node.test
            if isinstance(test, ast.Compare):
                if decide_compare_complicate_truth_value(test):
                    new_code = transform_truth_value_test_compli_to_simple.transform_c_s_truth_value_test(test)
                    code_list.append([test,new_code])


        elif isinstance(node, ast.BoolOp):
            for value in node.values:
                if isinstance(value, ast.Compare):
                    if decide_compare_complicate_truth_value(value):
                        new_code = transform_truth_value_test_compli_to_simple.transform_c_s_truth_value_test(value)
                        code_list.append([value,new_code])

    # for e in code_list:
    #     print("code1: ", e)
    return code_list
def get_truth_value_test_code(content):
    code_pair_list = []
    # print(content)
    try:

        file_tree = ast.parse(content)
        ana_py = ast_util.Fun_Analyzer()
        ana_py.visit(file_tree)
        # print("come here", ana_py.func_def_list)
        for tree, class_name in ana_py.func_def_list:
            try:
                me_name = tree.name
            except:
                me_name = ""
            # print("come method", ast.unparse(tree))
            new_code_list=get_truth_value_test(tree)
            # print("come method", new_code_list,ast.unparse(tree))
            for ind, (old_node,new_node) in enumerate(new_code_list):
                line_list=[[(old_node.lineno,old_node.col_offset), (old_node.end_lineno,old_node.end_col_offset)]]
                old_node_content = ast.get_source_segment(content, old_node)

                code_pair_list.append([class_name,me_name,[old_node_content], [ast.unparse(new_node)],line_list])
        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list

def get_part_truth_value_test_code(content):
    code_pair_list = []
    # print(content)
    try:

        tree = ast.parse(content)

        # print("come here", ana_py.func_def_list)
        if 1:#for tree, class_name in ana_py.func_def_list:
            try:
                me_name = tree.name
            except:
                me_name = ""
            # print("come method", ast.unparse(tree))
            new_code_list=get_truth_value_test(tree)
            # print("come method", new_code_list,ast.unparse(tree))
            for ind, (old_node,new_node) in enumerate(new_code_list):
                line_list=[[(old_node.lineno,old_node.col_offset), (old_node.end_lineno,old_node.end_col_offset)]]
                old_node_content = ast.get_source_segment(content, old_node)

                code_pair_list.append(['',me_name,[old_node_content], [ast.unparse(new_node)],line_list])
        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list
# def save_repo_for_else_complicated(repo_name):
#     start=time.time()
#     count_complicated_code = 0
#     print("come the repo: ", repo_name)
#     one_repo_for_else_code_list = []
#
#     dict_file=dict()
#     # if os.path.exists(save_complicated_code_dir_pkl + repo_name + ".pkl"):
#     #     print("the repo has been saved before")
#     #     return None
#     if 1:#not os.path.exists(save_complicated_code_dir_pkl + repo_name + ".pkl"):
#         for ind,file_info in enumerate(dict_repo_file_python[repo_name]):
#
#             file_path = file_info["file_path"]
#             # if file_path!="/mnt/zejun/smp/data/python_repo_1000/VideoPose3D//run.py":
#             #     continue
#             file_html = file_info["file_html"]
#             # if file_html != "https://github.com/google/TensorNetwork/tree/master/tensornetwork/block_sparse/blocksparsetensor.py":  # "https://github.com/tanghaibao/goatools/tree/master/goatools/go_enrichment.py":#"https://github.com/microsoft/msticpy/tree/master/msticpy/sectools/syslog_utils.py":#"https://github.com/chezou/tabula-py/tree/master/tabula/io.py":#"https://github.com/astorfi/speechpy/tree/master/speechpy/feature.py":#"https://github.com/pytoolz/toolz/tree/master/toolz/functoolz.py":#"https://github.com/smicallef/spiderfoot/tree/master//sfwebui.py":#"https://github.com/huggingface/transformers/tree/master/src/transformers/trainer_pt_utils.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/core.py":#"https://github.com/google/yapf/tree/master/yapf/yapflib/file_resources.py":#"https://github.com/zalando/patroni/tree/master/patroni/ctl.py":#"https://github.com/pymc-devs/pymc/tree/master/pymc/sampling.py":#"https://github.com/smicallef/spiderfoot/tree/master//sflib.py":#"https://github.com/smicallef/spiderfoot/tree/master//sfwebui.py":#
#             #     continue
#             # if file_html != "https://github.com/chezou/tabula-py/tree/master/tabula/io.py":  # "https://github.com/astorfi/speechpy/tree/master/speechpy/feature.py":#"https://github.com/pytoolz/toolz/tree/master/toolz/functoolz.py":#"https://github.com/smicallef/spiderfoot/tree/master//sfwebui.py":#"https://github.com/huggingface/transformers/tree/master/src/transformers/trainer_pt_utils.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/core.py":#"https://github.com/google/yapf/tree/master/yapf/yapflib/file_resources.py":#"https://github.com/zalando/patroni/tree/master/patroni/ctl.py":#"https://github.com/pymc-devs/pymc/tree/master/pymc/sampling.py":#"https://github.com/smicallef/spiderfoot/tree/master//sflib.py":#"https://github.com/smicallef/spiderfoot/tree/master//sfwebui.py":#
#             #     continue
#             # if file_html!="":#"https://github.com/astorfi/speechpy/tree/master/speechpy/feature.py":#"https://github.com/vt-vl-lab/3d-photo-inpainting/tree/master//mesh.py":
#             #     continue
#
#             try:
#                 content = util.load_file_path(file_path)
#             except:
#                 print(f"{file_path} is not existed!")
#                 continue
#
#             # print("content: ",content)
#             try:
#                 # print("come here")
#                 file_tree = ast.parse(content)
#                 ana_py = ast_util.Fun_Analyzer()
#                 ana_py.visit(file_tree)
#                 # print("ana_py.func_def_list ", ana_py.func_def_list)
#                 # dict_file["repo_name"]=repo_name
#
#
#                 dict_class=dict()
#                 for tree, class_name in ana_py.func_def_list:
#
#                     new_code_list=get_truth_value_test(tree)
#                     # print("new_code_list ", new_code_list)
#                     #
#                     # for old_code,new_code in new_code_list:
#                     #      print("old_code,new_code: ",ast.unparse(old_code),ast.unparse(new_code))
#                     ast_util.set_dict_class_code_list(tree,dict_class, class_name, new_code_list)
#
#
#                 dict_file[file_html]=dict_class
#
#
#
#             except SyntaxError:
#                 print("the file has syntax error")
#                 continue
#             except ValueError:
#                 print("the file has value error: ", content, file_html)
#                 continue
#             # break
#         end = time.time()
#         #'''
#         if dict_file:
#             count_complicated_code = count_complicated_code + len(one_repo_for_else_code_list)
#             # print("it exists for else complicated code1: ", len(one_repo_for_else_code_list))
#             util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
#             # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
#             print(end-start," save successfully! ", save_complicated_code_dir_pkl + repo_name)
#         else:
#             print(end-start," the repo has no for else")
#             util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
#             # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
#         #'''
#         # return count_complicated_code
if __name__ == '__main__':
    code='''
if self . get_conf_value ( 'show' , header = header ) == [ ] :
    pass
elif stats_grab == { } :
    pass
assert policy . remember ( pretend . stub ( ) , pretend . stub ( ) ) == 0
while a!=[]:
    pass

'''



    # ana_py = Analyzer()
    # ana_py.visit(file_tree)
    # print("fun number: ",len(ana_py.func_def_list))

    # save_complicated_code_dir= util.data_root + "complicated_code_dir/truth_value_test_complicated/"
    # save_complicated_code_dir_pkl= util.data_root + "transform_complicate_to_simple_pkl/truth_value_test_complicated_remove_is_is_not/"
    # #dict_repo_file_python=util.load_json(util.data_root, "python3_repos_files_info" )
    # # dict_repo_file_python=util.load_json(util.data_root, "python3_1000repos_files_info_modify" )
    # dict_repo_file_python= util.load_json(util.data_root, "python3_1000repos_files_info")
    #
    #
    # repo_name_list=[]
    # for repo_name in dict_repo_file_python:
    #     # if repo_name!="3d-photo-inpainting":#anki
    #     #     continue
    #     # if repo_name!="TensorNetwork":#"tabula-py":#"speechpy":#"gdb-dashboard":
    #     #     continue
    #     if os.path.exists(save_complicated_code_dir_pkl + repo_name + ".pkl"):
    #
    #         continue
    #     repo_name_list.append(repo_name)
    # # save_repo_for_else_complicated(repo_name_list[0])
    # print("len of repo: ",len(repo_name_list))
    # '''
    # count_complicated_code=0
    # for ind,repo_name in enumerate(dict_repo_file_python):
    #     one_repo_truth_value_test_code_list = []
    #     for file_info in dict_repo_file_python[repo_name]:
    #         file_path = file_info["file_path"]
    #         file_html = file_info["file_html"]
    #         content = util.load_file(file_path)
    #         try:
    #         #if 1:
    #             one_file_truth_value_test_code_list=get_truth_value_test(content)
    #             count_complicated_code+=len(one_file_truth_value_test_code_list)
    #             if one_file_truth_value_test_code_list:
    #                 one_repo_truth_value_test_code_list.append([one_file_truth_value_test_code_list, file_path, file_html])
    #                 # print("one_file_truth_value_test_code_list: ",one_file_truth_value_test_code_list)
    #                 # break
    #         except SyntaxError:
    #             print("the file has syntax error")
    #             continue
    #         # break
    #     if one_repo_truth_value_test_code_list:
    #         # print("it exists truth value test complicated code1: ", len(one_repo_for_else_code_list))
    #         util.save_json(save_complicated_code_dir, repo_name, one_repo_truth_value_test_code_list)
    #
    #     # break
    # # print()
    #
    #
    #     # break
    # print("count_complicated_code: ",ind,count_complicated_code)
    #
    # '''
    # '''
    # pool = newPool(nodes=30)
    # pool.map(save_repo_for_else_complicated, repo_name_list)  # [:3]sample_repo_url ,token_num_list[:1]
    # pool.close()
    # pool.join()
    # # print("len all_files: ", len(all_files))
    # '''
    #
    # files_num_list = []
    # star_num_list = []
    # contributor_num_list = []
    # count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    # file_list = set([])
    # repo_code_num = dict()
    # result_compli_for_else_list = []
    # all_count_repo, all_file_count, all_me_count = 0, 0, 0
    # for file_name in os.listdir(save_complicated_code_dir_pkl):
    #     all_count_repo += 1
    #     repo_name = file_name[:-4]
    #     # files_num_list.append(repo_files_info[repo_name])
    #     # star_num_list.append(repo_star_info[repo_name])
    #     # contributor_num_list.append(repo_contributor_info[repo_name])
    #
    #     complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)
    #
    #     repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
    #         complicate_code)
    #     # for code_list, file_path, file_html in complicate_code:
    #     code_count += repo_code_count
    #     file_count += repo_file_count
    #     me_count += repo_me_count
    #     all_file_count += repo_all_file_count
    #     all_me_count += repo_all_me_count
    #     repo_exist = 0
    #     for file_html in complicate_code:
    #         for cl in complicate_code[file_html]:
    #             for me in complicate_code[file_html][cl]:
    #                 if complicate_code[file_html][cl][me]:
    #                     repo_exist = 1
    #                     for code in complicate_code[file_html][cl][me]:
    #                         # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
    #                         #                code_index_start_end_list.append([node,assign_stmt,node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])
    #
    #                         result_compli_for_else_list.append(
    #                             [repo_name, file_html, cl, me, code[0].lineno,ast.unparse(code[0]), ast.unparse(code[1])])
    #
    #         # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
    #     count_repo += repo_exist
    #
    # # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # # print(a)
    # # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    # import random
    #
    # random.shuffle(result_compli_for_else_list)
    # util.save_csv(util.data_root + "result_csv/truth_value_test.csv", result_compli_for_else_list[:400],
    #               ["repo_name", "file_html", "class_name", "me_name", "line_no", "old_code", "new_code"])
    #
    # print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    # # 1 156 2943 100 1 2990 40102 salt
    # # 791 2010 103165 1291 800 121348 1192868
    # # util.save_csv(util.data_root + "transform_complicate_to_simple_pkl/truth_value_test_complicated_remove_is_isnot.csv", result_compli_for_else_list,
    # #               ["repo_name", "file_html", "class_name", "me_name", "for_code", "assign_code"])
    #
    #




    # #'''
    # count=0
    # result_compli_for_else_list=[]
    # for file_name in os.listdir(save_complicated_code_dir):
    #     complicate_code=util.load_json(save_complicated_code_dir,file_name[:-5])
    #     for code_list, file_path,file_html in complicate_code:
    #         count += len(code_list)
    #         for code1 in code_list:
    #             repo_name=file_html.split("/")[4]
    #             result_compli_for_else_list.append([repo_name,code1,file_html,file_path])
    #         #     print("one code1: ",repo_name,code1,file_html,file_path)
    #         #     break
    #         # break
    #     # print("file: ",file_name)
    #     # break
    # print("count: ",count,len(os.listdir(save_complicated_code_dir)))
    # util.save_csv(util.data_root+"complicated_code_dir/truth_value_test_complicated.csv",result_compli_for_else_list,["repo_name","code1","file_html","file_path"])
    #'''











    # print("----------------------------\n")
    # for code1 in for_else_filter_redundant_code_list:
    #     print("each code1: ", code1[0])



import time

import sys, ast, os, copy
import tokenize
import sys,shutil
import util
def get_code_count(complicate_code):
    all_me_count = 0
    all_file_count = len(list(complicate_code.keys()))
    code_count = 0
    me_count = 0
    NULL_code_count = 0
    file_count = 0
    cl_count = 0
    for file_html in complicate_code:

        if complicate_code[file_html]:
            file_exist = 0
            # cl_count +=len(list(complicate_code[file_html].keys()))
            for cl in complicate_code[file_html]:
                if complicate_code[file_html][cl]:
                    all_me_count += len(complicate_code[file_html][cl].keys())
                    for me in complicate_code[file_html][cl]:
                        if complicate_code[file_html][cl][me]:
                            me_count += 1
                            file_exist = 1
                            code_count += len(complicate_code[file_html][cl][me])

            file_count += file_exist

    return file_count, me_count, code_count, all_file_count, all_me_count


def get_code_count_contain_test(complicate_code):
    all_me_count = 0
    all_file_count = len(list(complicate_code.keys()))
    code_count,me_count,file_count = 0,0,0


    file_no_test_count, me_no_test_count, code_no_test_count=0,0,0

    for file_html in complicate_code:
        no_test_flag=0
        map_file_name = file_html.split("/")[-1][:-3]
        # filter out test files
        if not (map_file_name.startswith("test_") or map_file_name.endswith("_test")):
            no_test_flag=1
        # file_no_test_count+=no_test_flag

        if complicate_code[file_html]:
            test_file_exist=0
            file_exist = 0
            # cl_count +=len(list(complicate_code[file_html].keys()))
            for cl in complicate_code[file_html]:
                if complicate_code[file_html][cl]:
                    all_me_count += len(complicate_code[file_html][cl].keys())
                    for me in complicate_code[file_html][cl]:
                        if complicate_code[file_html][cl][me]:
                            if no_test_flag:
                                test_file_exist=1
                                me_no_test_count+=1
                                code_no_test_count += len(complicate_code[file_html][cl][me])

                            me_count += 1
                            file_exist = 1
                            code_count += len(complicate_code[file_html][cl][me])

            file_count += file_exist
            file_no_test_count+=test_file_exist

    return file_count, me_count, code_count, all_file_count, all_me_count, file_no_test_count, me_no_test_count, code_no_test_count
def get_total_name(file_html,cl,me_name):
    map_file_name = file_html.split("/")[-1][:-3]
    real_file_html = file_html.replace("//", "/")
    rela_path = ".".join(real_file_html.split("/")[6:-1])

    if rela_path:
        total_name = ".".join([rela_path, map_file_name, cl, me_name])
    else:
        total_name = ".".join([map_file_name, cl, me_name])
    total_name = total_name.replace("..", ".")
    return total_name
def get_code_info(complicate_code,repo_name):
    repo_list=set([])
    file_list = set([])
    me_list=set([])
    code_count=0
    for file_html in complicate_code:

        if complicate_code[file_html]:

            for cl in complicate_code[file_html]:
                if complicate_code[file_html][cl]:
                    for me in complicate_code[file_html][cl]:
                        if complicate_code[file_html][cl][me]:

                            me_list.add(get_total_name(file_html,cl,me))
                            file_list.add(file_html)
                            code_count += len(complicate_code[file_html][cl][me])
    if file_list:
        repo_list.add(repo_name)
    return  repo_list,file_list,me_list,code_count
def get_code_info_contain_test(complicate_code,repo_name):
    code_no_test_count=0
    repo_name_list,file_no_test_list, me_no_test_list=set([]),set([]),set([])
    for file_html in complicate_code:
        no_test_flag=0
        map_file_name = file_html.split("/")[-1][:-3]
        # filter out test files
        if not (map_file_name.startswith("test_") or map_file_name.endswith("_test")):
            no_test_flag=1
        if complicate_code[file_html]:
            for cl in complicate_code[file_html]:
                if complicate_code[file_html][cl]:
                    for me in complicate_code[file_html][cl]:
                        if complicate_code[file_html][cl][me]:
                            if no_test_flag:
                                file_no_test_list.add(file_html)
                                me_no_test_list.add(get_total_name(file_html, cl, me))
                                code_no_test_count += len(complicate_code[file_html][cl][me])
    if file_no_test_list:
        repo_name_list.add(repo_name)

    return repo_name_list, file_no_test_list, me_no_test_list, code_no_test_count
def get_code_info_with_test_case(dict_comp_file, repo_name):
    repo_name_list, file_no_test_list, me_no_test_list, code_no_test = set([]), set([]), set([]), 0

    # dict_comp_file = util.load_pkl(save_me_test_me_dir, repo_name)
    for file_html in dict_comp_file:
            file_no_test_list.add(file_html)
            for full_me in dict_comp_file[file_html]:
                me_no_test_list.add(full_me)
                code_no_test += dict_comp_file[file_html][full_me]["complica_num"]
    if   code_no_test:
        repo_name_list.add(repo_name)
    return repo_name_list, file_no_test_list, me_no_test_list, code_no_test

def get_code_info_with_success_test_case(dict_test_acc, repo_name):
        repo_name_list, file_no_test_list, me_no_test_list, code_count = set([]), set(dict_test_acc['file_list']), set(
            dict_test_acc['me_list']), dict_test_acc['total_count']
        if code_count:
            repo_name_list.add(repo_name)
        return repo_name_list, file_no_test_list, me_no_test_list, code_count
def get_comp_code_can_test_me(repo_name,save_test_file_res_dir,save_me_test_me_dir):
    dict_complica_me_list = dict()
    full_test_me_list = []
    dict_test_html_each_me_res = util.load_pkl(save_test_file_res_dir, repo_name)
    dict_intersect_test_methods = util.load_pkl(save_me_test_me_dir, repo_name)

    for test_html in dict_test_html_each_me_res:
        if dict_test_html_each_me_res[test_html]:
            full_test_me_list.extend(dict_test_html_each_me_res[test_html]['test_methods'])

    for file_html in dict_intersect_test_methods:
        dict_complica_me_list[file_html]=dict()
        for complic_me in dict_intersect_test_methods[file_html]:
            dict_complica_me_list[file_html][complic_me] = []
            dict_test_info = dict()
            # print(complic_me, dict_intersect_test_methods[file_html][complic_me])
            complica_code_fragments_num = dict_intersect_test_methods[file_html][complic_me]['complica_num']
            test_me_of_compl_code = []
            for test_case_html, cl, me in dict_intersect_test_methods[file_html][complic_me]["test_pair"]:
                real_file_html = test_case_html.replace("//", "/")
                path_list = real_file_html.split("/")[6:]
                path_list[-1] = path_list[-1][:-3]
                rela_path = ".".join(path_list)

                if cl:
                    total_test_me_name = ".".join([rela_path, cl, me])
                else:
                    total_test_me_name = ".".join([rela_path, me])
                test_me_of_compl_code.append(total_test_me_name)
                dict_test_info[total_test_me_name] = [test_case_html, rela_path, cl, me]
            for e in set(test_me_of_compl_code) & set(full_test_me_list):
                dict_complica_me_list[file_html][complic_me].append(dict_test_info[e])
    return dict_complica_me_list
def get_code_info_with_can_run_test_case(repo_name,save_test_file_res_dir,save_me_test_me_dir,complicated_code_dir_pkl):
    repo_name_list, file_no_test_list, me_no_test_list, code_count = set([]), set([]), set([]),0

    me_count=0
    file_count=0
    complic_code_count=0
    repo_fla = 0
    # if repo_name != "cloud-custodian":#"spiderfoot":
    #     continue
    # print("come to the repo: ", repo_name)
    dict_complica_me_list = get_comp_code_can_test_me(repo_name,save_test_file_res_dir,save_me_test_me_dir)
    # print("dict_complica_me_list: ", dict_complica_me_list)
    complicate_code = util.load_pkl(complicated_code_dir_pkl, repo_name)
    for file_html in complicate_code:
        flag_file = 0
        if file_html in dict_complica_me_list:

            # if file_html!="https://github.com/pymc-devs/pymc/tree/master/pymc/sampling.py":#"https://github.com/smicallef/spiderfoot/tree/master//sflib.py":#"https://github.com/smicallef/spiderfoot/tree/master//sfwebui.py":#
            #     continue

            # print("come the file_html: ", file_html)
            for cl in complicate_code[file_html]:

                for me in complicate_code[file_html][cl]:
                    me_name = me.split("$")[0]
                    if me_name == "if_main_my":  # it is impossible for the main code1 have test cases
                        continue
                    total_name = get_total_name(file_html, cl, me_name)
                    # print("come total_name: ", cl,me_name,total_name)
                    if total_name in dict_complica_me_list[file_html]:
                        test_me_inf_list = dict_complica_me_list[file_html][total_name]
                        if complicate_code[file_html][cl][me] and test_me_inf_list:
                            repo_name_list.add(repo_name)
                            file_no_test_list.add(file_html)
                            me_no_test_list.add(total_name)
                            flag_file = 1
                            repo_fla = 1
                            me_count+= 1
                            complic_code_count += len(complicate_code[file_html][cl][me])

        file_count += flag_file

    # return repo_fla,file_count,me_count,complic_code_count
    return repo_name_list,file_no_test_list,me_no_test_list,complic_code_count








import sys,ast, copy,os

abs_path=os.path.abspath(os.path.dirname(__file__))
# print("abs_path: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
# print(pack_path)
sys.path.append(pack_path)
import util


def copy_compre(node):
    compr = ast.comprehension()
    compr.target = node.target
    compr.iter = node.iter
    compr.ifs=[]
    compr.is_async=None
    # print("compr: ",ast.unparse(compr))
    return compr


def for_transform(node,listcompre):
    # print("node: ",node,ast.unparse(node))

    if isinstance(node,ast.For):
        body = node.body[0]
        listcompre.generators.append(copy_compre(node))

        for_transform(body,listcompre)
        # print("come to the ast.For hahahha: ",ast.unparse(listcompre))

    elif isinstance(node,ast.If):
        body = node.body[0]
        if not node.orelse:
            listcompre.generators[-1].ifs.append(node.test)
            for_transform(body, listcompre)
            pass
        else:
            def if_else_transform(node,listcompre):
                if isinstance(node,ast.If):
                    ifexpr = ast.IfExp()
                    ifexpr.test = node.test
                    ifexpr.body = node.body[0].value
                    listcompre.key=node.body[0].targets[0].slice
                    ifexpr.orelse = if_else_transform(node.orelse[0],listcompre)
                    return ifexpr
                else:
                    return node.value



            listcompre.value =if_else_transform(node,listcompre)
            # ifexpr=ast.IfExp()
            # ifexpr.test=node.test
            #
            # ifexpr.body=body.value.args[0]
            # ifexpr.orelse=None
            # for_transform(node.orelse[0], ifexpr.orelse)
            #
            # # ifexpr.orelse=listcompre.elt
            # listcompre.elt = ifexpr

    else:
        # print("node:",ast.unparse(node))
        listcompre.key = node.targets[0].slice
        listcompre.value = node.value


def test_fun(a):
    a.id="change"
    print()
def transform(for_node, assign_node):
    new_code=copy.deepcopy(assign_node)

    listcompre=ast.DictComp()
    listcompre.generators=[]
    listcompre.key=None
    listcompre.value = None
    for_transform(for_node,listcompre)
    new_code.value = listcompre
    return new_code
def replace_file_content(for_node,assign_stmt,new_code,real_file_html,repo_name):
    rela_path = "/".join(real_file_html.split("/")[6:])
    file_path = "".join([util.data_root, "python_star_2000repo/", repo_name, "/", rela_path])
    content = util.load_file_path(file_path)
    res_copy = content.split("\n")
    indent = ""
    for ind, e in enumerate(res_copy[for_node.lineno - 1]):
        if e != " ":
            indent = " " * ind
            break

    res_copy[assign_stmt.lineno - 1] = indent + ast.unparse(new_code)
    res_copy[for_node.lineno - 2:for_node.end_lineno] = res_copy[for_node.lineno - 2:for_node.lineno - 1]
    return content,"\n".join(res_copy)
def traverse(node, complicate_code, assign_str):
    if isinstance(node, ast.For):
        target = node.target
        iter = node.iter
        body = node.body[0]
        complicate_code.append("for " + ast.unparse(target) + " in " + ast.unparse(iter) + " ")
        traverse(body, complicate_code, assign_str)



    elif isinstance(node, ast.If):
        test = node.test
        body = node.body[0]
        if not node.orelse:

            complicate_code.append("if " + ast.unparse(test) + " ")
            traverse(body, complicate_code, assign_str)
        else:  # 这个是一个赋值语句
            if isinstance(body, ast.Expr) and isinstance(body.value, ast.Call):
                args_if = ast.unparse(body.value.args)

            or_else = node.orelse[0]
            if isinstance(or_else, ast.Expr) and isinstance(or_else.value, ast.Call):
                args_or_else = ast.unparse(or_else.value.args)

                assign_str.append(args_if + " if " + ast.unparse(test) + " else " + args_or_else + " ")
            else:
                assign_str.append(args_if + " if " + ast.unparse(test) + " else ")
                traverse(or_else, complicate_code, assign_str)
            # print(args_or_else)
            # complicate_code.insert(0,args_if+" if " + ast.unparse(test) + " else "+args_or_else+" ")

    elif isinstance(node, ast.Expr) and isinstance(node.value, ast.Call):
        assign_str.append(ast.unparse(node.value.args) + " ")
        # complicate_code.insert(0,args_assign_expr)
if __name__ == '__main__':
    code='''
for (key, value) in input_dict.items():
    if 'rgb' in key or 'color' in key:
        rt_dict[key] = torch.FloatTensor(value).permute(2, 0, 1)[None, ...]
    else:
        rt_dict[key] = torch.FloatTensor(value)[None, None, ...]
'''
    tree= ast.parse(code)
    for_node = None
    ass_node = None
    for node in ast.walk(tree):
        if isinstance(node, ast.Assign):
            ass_node = node
        elif isinstance(node, ast.For):
            for_node = node

    new_code = transform(for_node, ass_node)
    print("new_code: ", ast.unparse(new_code))
    # complicate_code=[]
    # transform_node=[]
    # Var=""
    # #'''
    # complicate_code=[]
    # assign_left_code=[]
    # for node in ast.walk(tree):
    #     if isinstance(node,ast.Assign):
    #         left=node.targets
    #         Var=ast.unparse(left)
    #         assign_left_code.append(Var+" = [ ")
    #
    #
    #     elif isinstance(node,ast.For):
    #         assign_str=[]
    #         traverse(node, complicate_code,assign_str)
    #         complicate_code.insert(0,"".join(assign_str))
    #         complicate_code.append("]")
    #         break
    # assign_left_code.extend(complicate_code)
    # str_complicate_code="".join(assign_left_code)
    # print(str_complicate_code)
    #'''

















import sys,ast,os
abs_path=os.path.abspath(os.path.dirname(__file__))

# abs_path=os.getcwd()
# print("abs_path: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
# print(pack_path)
sys.path.append(pack_path)
# from RefactoringIdioms  import util
import time,complicated_code_util,util
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
from RefactoringIdioms.transform_c_s import transform_truth_value_test_compli_to_simple

# from code1 import util
# from code1.transform_c_s import transform_truth_value_test_compli_to_simple
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
from pathos.multiprocessing import ProcessingPool as newPool

'''
input: 代码片段 code1
if if,while,assert节点
    if test是compare节点
        
elif if boolOP节点
    if values 中含有compare 节点


'''
def decide_compare_complicate_truth_value(test):
    flag=0
    ops = test.ops
    left = ast.unparse(test.left)
    comparators = test.comparators
    comp_str = ast.unparse(comparators[0])

    empty_list = ["None", "True", "False", "0", "0.0", "0j", "Decimal(0)", "Fraction(0, 1)", '', "()","[]", "{}", "dict()", "set()", "range(0)"]

    if len(ops) == 1 and isinstance(ops[0], (ast.Eq, ast.NotEq, ast.Is, ast.IsNot)):
        if left in empty_list or comp_str in empty_list:
            flag=1
    return flag
def get_truth_value_test(tree):
    code_list = []

    for node in ast.walk(tree):

        if isinstance(node, (ast.If, ast.While, ast.Assert)):
            test = node.test
            if isinstance(test, ast.Compare):
                if decide_compare_complicate_truth_value(test):
                    new_code = transform_truth_value_test_compli_to_simple.transform_c_s_truth_value_test(test)
                    code_list.append([test,new_code])


        elif isinstance(node, ast.BoolOp):
            for value in node.values:
                if isinstance(value, ast.Compare):
                    if decide_compare_complicate_truth_value(value):
                        new_code = transform_truth_value_test_compli_to_simple.transform_c_s_truth_value_test(value)
                        code_list.append([value,new_code])

    # for e in code_list:
    #     print("code1: ", e)
    return code_list
def save_repo_for_else_complicated(repo_name):
    start=time.time()
    count_complicated_code = 0
    print("come the repo: ", repo_name)
    one_repo_for_else_code_list = []

    dict_file=dict()
    # if os.path.exists(save_complicated_code_dir + repo_name + ".json"):
    #     print("the repo has been saved before")
    #     return None

    for ind,file_info in enumerate(dict_repo_file_python[repo_name]):



        file_path = file_info["file_path"]
        # if file_path!="/mnt/zejun/smp/data/python_repo_1000/VideoPose3D//run.py":
        #     continue
        file_html = file_info["file_html"]
        # if file_html!="https://github.com/vt-vl-lab/3d-photo-inpainting/tree/master//mesh.py":
        #     continue

        try:
            content = util.load_file_path(file_path)
        except:
            print(f"{file_path} is not existed!")
            continue

        # print("content: ",content)
        try:
            # print("come here")
            file_tree = ast.parse(content)
            ana_py = ast_util.Fun_Analyzer()
            ana_py.visit(file_tree)
            # print("ana_py.func_def_list ", ana_py.func_def_list)
            # dict_file["repo_name"]=repo_name


            dict_class=dict()
            for tree, class_name in ana_py.func_def_list:

                new_code_list=get_truth_value_test(tree)
                # print("new_code_list ", new_code_list)
                #
                # for old_code,new_code in new_code_list:
                #     print("old_code,new_code: ",ast.unparse(old_code),ast.unparse(new_code))
                ast_util.set_dict_class_code_list(tree, dict_class, class_name, new_code_list)


            dict_file[file_html]=dict_class



        except SyntaxError:
            print("the file has syntax error")
            continue
        except ValueError:
            print("the file has value error: ", content, file_html)
            continue
        # break
    end = time.time()
    #'''
    if dict_file:
        count_complicated_code = count_complicated_code + len(one_repo_for_else_code_list)
        # print("it exists for else complicated code1: ", len(one_repo_for_else_code_list))
        util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
        # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
        print(end-start," save successfully! ", save_complicated_code_dir_pkl + repo_name)
    else:
        print(end-start," the repo has no for else")
        util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
        # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
    #'''
    return count_complicated_code
if __name__ == '__main__':
    code='''
if self . get_conf_value ( 'show' , header = header ) == [ ] :
    pass
elif stats_grab == { } :
    pass
assert policy . remember ( pretend . stub ( ) , pretend . stub ( ) ) == 0
while a!=[]:
    pass

'''



    # ana_py = Analyzer()
    # ana_py.visit(file_tree)
    # print("fun number: ",len(ana_py.func_def_list))

    save_complicated_code_dir= util.data_root + "complicated_code_dir/truth_value_test_complicated/"
    save_complicated_code_dir_pkl= util.data_root + "transform_complicate_to_simple_pkl/truth_value_test_complicated/"
    #dict_repo_file_python=util.load_json(util.data_root, "python3_repos_files_info" )
    # dict_repo_file_python=util.load_json(util.data_root, "python3_1000repos_files_info_modify" )
    dict_repo_file_python= util.load_json(util.data_root, "python3_1000repos_files_info")


    repo_name_list=[]
    for repo_name in dict_repo_file_python:
        # if repo_name!="3d-photo-inpainting":
        #     continue
        repo_name_list.append(repo_name)
    # save_repo_for_else_complicated(repo_name_list[0])
    '''
    count_complicated_code=0
    for ind,repo_name in enumerate(dict_repo_file_python):
        one_repo_truth_value_test_code_list = []
        for file_info in dict_repo_file_python[repo_name]:
            file_path = file_info["file_path"]
            file_html = file_info["file_html"]
            content = util.load_file(file_path)
            try:
            #if 1:
                one_file_truth_value_test_code_list=get_truth_value_test(content)
                count_complicated_code+=len(one_file_truth_value_test_code_list)
                if one_file_truth_value_test_code_list:
                    one_repo_truth_value_test_code_list.append([one_file_truth_value_test_code_list, file_path, file_html])
                    # print("one_file_truth_value_test_code_list: ",one_file_truth_value_test_code_list)
                    # break
            except SyntaxError:
                print("the file has syntax error")
                continue
            # break
        if one_repo_truth_value_test_code_list:
            # print("it exists truth value test complicated code1: ", len(one_repo_for_else_code_list))
            util.save_json(save_complicated_code_dir, repo_name, one_repo_truth_value_test_code_list)

        # break
    # print()


        # break
    print("count_complicated_code: ",ind,count_complicated_code)
    
    '''
    #'''
    pool = newPool(nodes=30)
    pool.map(save_repo_for_else_complicated, repo_name_list)  # [:3]sample_repo_url ,token_num_list[:1]
    pool.close()
    pool.join()
    # print("len all_files: ", len(all_files))
    #'''

    files_num_list = []
    star_num_list = []
    contributor_num_list = []
    count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    file_list = set([])
    repo_code_num = dict()
    result_compli_for_else_list = []
    all_count_repo, all_file_count, all_me_count = 0, 0, 0
    for file_name in os.listdir(save_complicated_code_dir_pkl):
        all_count_repo += 1
        repo_name = file_name[:-4]
        # files_num_list.append(repo_files_info[repo_name])
        # star_num_list.append(repo_star_info[repo_name])
        # contributor_num_list.append(repo_contributor_info[repo_name])

        complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)

        repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
            complicate_code)
        # for code_list, file_path, file_html in complicate_code:
        code_count += repo_code_count
        file_count += repo_file_count
        me_count += repo_me_count
        all_file_count += repo_all_file_count
        all_me_count += repo_all_me_count
        repo_exist = 0
        for file_html in complicate_code:
            for cl in complicate_code[file_html]:
                for me in complicate_code[file_html][cl]:
                    if complicate_code[file_html][cl][me]:
                        repo_exist = 1
                        for code in complicate_code[file_html][cl][me]:
                            # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
                            #                code_index_start_end_list.append([node,assign_stmt,node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])

                            result_compli_for_else_list.append(
                                [repo_name, file_html, cl, me, ast.unparse(code[0]), ast.unparse(code[1])])

            # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
        count_repo += repo_exist

    # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # print(a)
    # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    # 1 156 2943 100 1 2990 40102 salt
    # 791 2010 103165 1291 800 121348 1192868
    util.save_csv(util.data_root + "transform_complicate_to_simple_pkl/truth_value_test_complicated.csv", result_compli_for_else_list,
                  ["repo_name", "file_html", "class_name", "me_name", "for_code", "assign_code"])






    # #'''
    # count=0
    # result_compli_for_else_list=[]
    # for file_name in os.listdir(save_complicated_code_dir):
    #     complicate_code=util.load_json(save_complicated_code_dir,file_name[:-5])
    #     for code_list, file_path,file_html in complicate_code:
    #         count += len(code_list)
    #         for code1 in code_list:
    #             repo_name=file_html.split("/")[4]
    #             result_compli_for_else_list.append([repo_name,code1,file_html,file_path])
    #         #     print("one code1: ",repo_name,code1,file_html,file_path)
    #         #     break
    #         # break
    #     # print("file: ",file_name)
    #     # break
    # print("count: ",count,len(os.listdir(save_complicated_code_dir)))
    # util.save_csv(util.data_root+"complicated_code_dir/truth_value_test_complicated.csv",result_compli_for_else_list,["repo_name","code1","file_html","file_path"])
    #'''











    # print("----------------------------\n")
    # for code1 in for_else_filter_redundant_code_list:
    #     print("each code1: ", code1[0])




import json,os,csv,tokenize,token,pickle
root="/mnt/zejun/smp/"
data_root="/mnt/zejun/smp/data/"
code_root="/mnt/zejun/smp/code1/"
all_repos_dir="/mnt/zejun/smp/data/2021-4-20-github-top-10000stars/"
pro_path=data_root+"python_repo_1000/"
#root="/data/zzj/so/"#'/Users/sally/Downloads/xiaxin/third_html/dataset/Trans_Simpl_Compl/'
#data_dir=root+"data/"
test_case_list=[]
test_paras=[]
me_list=[]
import shutil
built_in=['abs','delattr','hash','memoryview','set','all','dict','help','min','setattr','any','dir','hex','next','slice','ascii','divmod','id','object','sorted','bin',
    'enumerate','input','oct','staticmethod','bool','eval','int','open','str','breakpoint','exec','isinstance','ord','sum','bytearray','filter','issubclass','pow','super',
    'bytes','float','iter','print','tuple','callable','format','len','property','type','chr','frozenset','list','range','vars','classmethod','getattr','locals','repr',
    'zip','compile','globals','map','reversed','__import__','complex','hasattr','max','round']
def mkdirs(save_for_else_methods_dir):
    if not os.path.exists(save_for_else_methods_dir):
        os.makedirs(save_for_else_methods_dir)
    else:
        shutil.rmtree(save_for_else_methods_dir)  # Removes all the subdirectories!
        os.makedirs(save_for_else_methods_dir)
def save_pkl(file_path,file_name,data):
    if not os.path.exists(file_path):
        os.makedirs(file_path)
    with open(file_path + file_name + '.pkl', 'wb') as file:
        return pickle.dump(data,file)
def load_pkl(file_path,file_name):
    with open(file_path +  file_name+'.pkl', 'rb') as file:
        return pickle.load(file)
def load_json(file_path,file_name):

    with open(file_path + file_name+'.json', 'r') as json_file:
        return json.load( json_file)
def save_json(file_path,file_name,data):
    if not os.path.exists(file_path):
        os.makedirs(file_path)
    with open(file_path + file_name+'.json', 'w') as json_file:
        json.dump(data, json_file)
def save_json_file_path(file_path,data):
    # if not os.path.exists(file_path):
    #     os.makedirs(file_path)
    with open(file_path, 'w') as json_file:
        json.dump(data, json_file)
def load_file(file_path,file_name):
    with open(file_path + file_name, 'rb') as file:

        return file.read()
def load_file_path(file_path):
    with open(file_path, 'r') as file:

        return file.read()
def load_file_path_lines(file_path):
    with open(file_path, 'r') as file:

        return file.readlines()
def save_file_path(file_path,content,format="",w="w"):
    path_dir="/".join(file_path.split("/")[:-1])

    if not os.path.exists(path_dir) and path_dir:
        os.makedirs(path_dir)
    with open(file_path, w) as file:

        file.write(content)
def save_file(file_path,file_name,content,format="",w="w"):
    path_dir="/".join(file_path.split("/")[:-1])
    if not os.path.exists(path_dir):
        os.makedirs(path_dir)
    with open(file_path + file_name+format, w) as file:

        file.write(content)
def save_csv(file_path,data, head=[]):
    dir_path="/".join(file_path.split("/")[:-1])
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)
    with open(file_path, 'w', encoding='utf-8', newline="") as file:
        writer = csv.writer(file)
        if head:
            writer.writerow(head)
        for item in data:
            writer.writerow(item)
def do_file(fname):
    """ Run on just one file.

    """
    source = open(fname)
    #mod = open(fname + ",strip", "w")
    new_str=""

    prev_toktype = token.INDENT
    first_line = None
    last_lineno = -1
    last_col = 0

    tokgen = tokenize.generate_tokens(source.readline)
    for toktype, ttext, (slineno, scol), (elineno, ecol), ltext in tokgen:
        if 0:   # Change to if 1 to see the tokens fly by.
            print("%10s %-14s %-20r %r" % (
                tokenize.tok_name.get(toktype, toktype),
                "%d.%d-%d.%d" % (slineno, scol, elineno, ecol),
                ttext, ltext
                ))
        if slineno > last_lineno:
            last_col = 0
        if scol > last_col:
            new_str+=" " * (scol - last_col)
            #mod.write(" " * (scol - last_col))
            pass
        if toktype == token.STRING and prev_toktype == token.INDENT:
            # Docstring
            #mod.write("#--")
            #new_str+="#--"
            pass
        elif toktype == tokenize.COMMENT:
            # Comment
           # mod.write("##\n")
            #new_str += "\n"
            pass
        else:
            #mod.write(ttext)
            new_str += ttext
        prev_toktype = toktype
        last_col = ecol
        last_lineno = elineno
    return new_str
def get_python3_repos(repo_path_dir):

    count=0
    flag=1
    for root,dirs,files in os.walk(repo_path_dir):
        #print(root)
        for file in files:
            if file.endswith(".py") and ("__init__" in file or "setup.py" in file):
                continue
            if file.endswith(".py"):
                parse_file_path = root + "/" + file


                try:
                    #content = load_file(parse_file_path)
                    content = do_file(parse_file_path)
                    # if "validate_rst_title_capitalization.py" in parse_file_path:
                    #     print("content: ",content)

                except:
                    print("the file do not be read!!!!!")
                    continue

                if "    print " in content or "\nprint " in content:
                    # print(parse_file_path)
                    # print(content)
                    flag=0
                    break

                count+=1
        if flag==0:
            break
    # print("num of python file in the repo: ",count)
    return flag,count

def load_csv(file_path):
    csvFile = open(file_path, "r")
    reader = csv.reader(csvFile)
    return list(reader)


class CodeInfo():
    def __init__(self, file_path="" ,idiom="",cl="",me="",
                 compli_code="", simple_cod="",lineno=[]):
        self.file_path = file_path
        self.cl = cl
        self.me = me
        self.idiom = idiom
        self.compli_code = compli_code
        self.simple_code = simple_cod
        self.lineno=lineno
        self.keyno=None
    # def lineno_str(self):
    #     linestr=""
    #     line_list=[]
    #     for e_lineno_start,e_lineno_end in self.lineno:
    #         if not line_list:
    #             line_list.append([e_lineno_start,e_lineno_end])
    #         else:
    #             if e_lineno_start-line_list[-1][-1]<=1:
    #                 line_list[-1][-1]=e_lineno_end
    #             else:
    #                 line_list.append([e_lineno_start, e_lineno_end])
    #     linestr=[]
    #     for start,end in line_list:
    #         linestr.append(f"lines {start} to {end}")
    #     return ", ".join(linestr)
    # def get_key_lineno(self):
    #     return float(f'{self.lineno.e_lineno_start}.{e_col_start}'),float(f'{e_lineno_end}.{e_col_end}')
    #
    def lineno_str(self):
        linestr=""
        line_list=[]
        for (e_lineno_start,e_col_start),(e_lineno_end,e_col_end) in self.lineno:
            # print("(e_lineno_start,e_col_start),(e_lineno_end,e_col_end): ",(e_lineno_start,e_col_start),(e_lineno_end,e_col_end))
            # linestr.append(f"lines {e_lineno_start}:{e_col_start}, {end}")
            line_list.append([[e_lineno_start, e_col_start], [e_lineno_end, e_col_end]])

            # if not line_list:
            #     line_list.append([[e_lineno_start,e_col_start],[e_lineno_end,e_col_end]])
            # else:
            #     if e_lineno_start-line_list[-1][-1][0]<=1:
            #         line_list[-1][-1][0]=e_lineno_end
            #     else:
            #         line_list.append([[e_lineno_start,e_col_start],[e_lineno_end,e_col_end]])

        linestr=[]
        for (start_lineno,start_colno),(end_lineno,end_colno) in line_list:
            linestr.append(f"lines {start_lineno}:{start_colno} to {end_lineno}:{end_colno}")
        return ", ".join(linestr)

    def full_info(self):
        return "***".join(["Filepath: "+self.file_path, "Class: "+self.cl if self.cl else '', "Method: "+self.me if self.me else '',"Idiom: "+self.idiom,self.code_str()])

    def code_str(self):
        # print("linenolist: ",self.lineno)
        # print("complicated code: ", self.compli_code)
        # print("simple code: ", self.simple_code)

        lineno_str=self.lineno_str()

        # if '' in self.compli_code:
        #     self.compli_code.remove('')
        # if '' in self.simple_code:
        #     self.simple_code.remove('')
        compli_code_list=[e for e in self.compli_code if e!='']
        simple_code_list=[e for e in self.simple_code if e!='']
        codestr=f"***{lineno_str}\n"
        return "".join([codestr,"\n".join(compli_code_list),"\n-----is refactored into----->\n","\n".join(simple_code_list)])
        # return "".join([codestr,"".join(self.compli_code),"\n-----is refactored into----->\n","".join(self.simple_code)])





import sys,ast,traceback,os

abs_path=os.path.abspath(os.path.dirname(__file__))
# print("abs_path: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
# print(pack_path)
sys.path.append(pack_path)



def create_list(num):
    # if num>1:
        a=[]
        for i in range(num-1):
            if a:
                a[-1].apppend([])
            else:
                a.append([])
        return a
def get_subscript_num(node):
    # count=0
    if isinstance(node.value,ast.Subscript):
        # print("count: ",count)
        return 1+get_subscript_num(node.value)
    else:
        return 1
    # return count
def get_subscript_index_not_last(node):
    # print("node: ",ast.unparse(node))
    if isinstance(node.value,ast.Subscript):
        a=[ast.unparse(node.slice)]
        a.extend(get_subscript_index_not_last(node.value))
        return a
    else:
        return [ast.unparse(node.slice)]


def build_node_list(node_list):

    pre_index=[]
    new_node_list=[]
    for ind, node in enumerate(node_list):
        cur_num=get_subscript_num(node)
        cur_index_list = get_subscript_index_not_last(node)[1:][::-1]
        last_ele = new_node_list
        list_num=cur_num-1
        for ind,cur_ind in enumerate(cur_index_list):
            if len(pre_index)>ind:
                if cur_ind!=pre_index[ind]:
                    break
                else:
                    list_num-=1
                    last_ele=last_ele[-1]
        for i in range(list_num):
            last_ele.append([])
            last_ele = last_ele[-1]


        last_ele.append(node)
        pre_index = cur_index_list
    return new_node_list
            # new list and then add it to the new list

def sort_node_list(node_list):
    map_dict=dict()
    node_str_list=[]
    for node in node_list:
        node_str=ast.unparse(node)
        node_str_list.append(node_str)
        map_dict[node_str]=node
    map_dict=sorted(map_dict.items(), key=lambda kv: (len(kv[0]),kv[0]))
    new_node_list=[]
    for e1,e2 in map_dict:
        new_node_list.append(e2)

    return build_node_list(new_node_list)





def build_pre_var_name(node):

    value = node.value
    var_name = ""
    if isinstance(value, ast.Subscript):
        var_name += build_pre_var_name(value)


    else:
        var_name = ast.unparse(value)

    return var_name
def get_beg(slice):
    if isinstance(slice,ast.Constant):
        return int(ast.unparse(slice)),int(ast.unparse(slice))
    elif isinstance(slice,ast.Slice):
        upper = int(ast.unparse(slice.upper)) if slice.upper else 'len'
        lower = int(ast.unparse(slice.lower)) if slice.lower else 0
        # step = ast.unparse(slice.step) if slice.step else '1'
        beg= lower

        return beg,upper
def add_node_before_beg(node,beg):
    pre_node=None
    cur_beg, cur_end = get_beg(node.slice)
    # print("beg: ",beg)
    # print("cur_beg, cur_end: ",cur_beg,beg, cur_end)
    if beg < cur_beg:
        pre_node_var_name = build_var(node).split("_")
        pre_node_var_name[-1] = str(cur_beg - 1)

        if cur_beg - beg > 1:
            pre_node = ast.Starred()
            pre_node.value = ast.Name("_".join(pre_node_var_name))

        else:
            pre_node = ast.Name("_".join(pre_node_var_name))
            # cur_end += 1
        # node_list.append(ast.unparse(pre_node))
    if cur_end == "len":
        print("the code1 need to be tested!")
        # break
        beg=cur_end
        # pre_node=None
        # return None,None,None
        # return beg, cur_beg, pre_node
    else:
        # beg = cur_end + 1
        beg = cur_end + 1
    return beg,cur_beg,pre_node
def get_slice(slice):
    if isinstance(slice,ast.Constant):
        return "_"+ast.unparse(slice)
    elif isinstance(slice,ast.Slice):
        upper=ast.unparse(slice.upper) if slice.upper else 'len'
        lower=ast.unparse(slice.lower) if slice.lower else '0'
        step=ast.unparse(slice.step) if slice.step else '1'

        return "_".join(["",lower,step,upper])
    else:
        return ""
def build_var(node):
    slice=node.slice
    value=node.value
    var_name=""
    if isinstance(value,ast.Subscript):
        var_name+=build_var(value)
        var_name+=get_slice(slice)

    else:
        var_name=ast.unparse(value)+get_slice(slice)
    return var_name
def add_cur_node(node):
    slice = node.slice
    var_name = build_var(node)
    if isinstance(slice, ast.Slice):
        var = ast.Starred()
        var.value = ast.Name(var_name)
    else:
        var = ast.Name(var_name)
    return var


def connect_slice_name(node):
    if isinstance(node, ast.Subscript):
        return [node]+connect_slice_name(node.value)
    else:
        return []
def get_first_node(node):
            if isinstance(node, list):

                return get_first_node(node[0])

            else:

                return node
def transform(node_list,new_list,depth,Map_var=None):
    beg=0
    last_node=None
    cannot_trans_list=[]
    star_count=0
    for node in node_list:
        if isinstance(node,list):
            first_node = get_first_node(node)

            cur_node = connect_slice_name(first_node)[::-1][depth - 1]
            # print("cur_node: ",ast.unparse(first_node),ast.unparse(cur_node),cur_node)
            beg, cur_beg,pre_node = add_node_before_beg(cur_node, beg)
            # if (not beg) and (not cur_beg) and (not pre_node):
            #     # cannot_trans_flag=1
            #     cannot_trans_list.append(node)
            #     new_list=[None]
            #     break


            if pre_node:
                if isinstance(pre_node,ast.Starred):
                    star_count+=1
                new_list.append(pre_node)
            new_list_2=[]
            the_cur_star_count=transform(node, new_list_2, depth+1,Map_var)
            if the_cur_star_count>1:
                return the_cur_star_count
            new_list.append(new_list_2)
        else:
            beg, cur_beg,pre_node = add_node_before_beg(node, beg )
            if pre_node:
                if isinstance(pre_node,ast.Starred):
                    star_count+=1

                new_list.append(pre_node)
            cur_node=add_cur_node(node)
            # Map_var[ast.unparse(cur_node)] = ast.unparse(node)
            Map_var[ast.unparse(node)] = cur_node

            new_list.append(cur_node)
            # new_list.append(ast.unparse(add_cur_node(node)))
    if beg!="len":
        # print("come here")
        fir_node = get_first_node(node_list[0])
        end_node = connect_slice_name(fir_node)[::-1][depth - 1]
        var_name=build_var(end_node).split("_")
        var_name[-1] = "len"
        last_node=ast.Starred()
        last_node.value=ast.Name("_".join(var_name))
        star_count += 1
        # new_list.append(ast.unparse(last_node))
        new_list.append(last_node)
    return star_count

def build_elts(new_node_list,tuple_list):
        for node in new_node_list:
            if isinstance(node,list):
                tuple = ast.Tuple()
                tuple.elts = []
                build_elts(node,tuple.elts )
                tuple_list.append(tuple)
            else:
                tuple_list.append(node)

class RewriteName(ast.NodeTransformer):
        def __init__(self,Map_var):
            self.Map_var=Map_var
        # def visit_Subscript(self, node):
        #     # print("visit_Subscript node: ", ast.unparse(node))
        #     if ast.unparse(node) in self.Map_var:
        #         # print("yes: ",ast.unparse(node))
        #         return self.Map_var[ast.unparse(node)]
        #     else:
        #         # node=RewriteName().visit(node)
        #         if isinstance(node.value,ast.Subscript):
        #             node.value=self.visit_Subscript(node.value)
        #         if isinstance(node.slice,ast.Subscript):
        #             node.slice = self.visit_Subscript(node.slice)
        #         return node
        def generic_visit(self, node):

            if ast.unparse(node) in self.Map_var:
                return self.Map_var[ast.unparse(node)]
            #
            for ind_field,k in enumerate(node._fields):
                # print("transfrom: ", k)
                try:

                # if 1:
                    v = getattr(node, k)
                    # print("here: ", k,v )
                    if isinstance(v,ast.AST):
                        if v._fields:
                            setattr(node, k, self.generic_visit(v))
                        # node._fields[k] = self.generic_visit(v)
                        pass
                    elif isinstance(v, list):

                        for ind,e in enumerate(v):
                            if hasattr(e,'_fields'):
                                v[ind]=self.generic_visit(e)
                        setattr(node, k, v)
                                # node._fields[k][ind]=self.generic_visit(e)
                                # pass
                except:
                    continue
                #     print("error: ", ast.unparse(v))
                #     if ast.unparse(v) in self.Map_var:
                #         setattr(node, k, self.Map_var[ast.unparse(v)])
                # #         # node._fields[k]= self.Map_var[ast.unparse(v)]
                #     continue
            #
            #
            return node


def transform_for_node_var_unpack(for_node,node_list,Map_var):
    try:
        node_list = sort_node_list(node_list)
        # print("node_list: ",node_list)
        # for node in node_list:
        #     print("node: ", ast.unparse(node))
        new_node_list = []

        star_count=transform(node_list, new_node_list, 1, Map_var)
        if star_count>1:
            return None
        tuple = ast.Tuple()
        tuple.elts = []
        build_elts(new_node_list, tuple.elts)
        # print("Map_var: ",Map_var)
        # for a in ast.walk(for_node):
        #     if ast.unparse(a) in Map_var:
        #         a = Map_var[ast.unparse(a)]
        #         print("yes: ", a)
        # print(ast.unparse(for_node))
        for_node.target = tuple
        # tree = ast.parse('foo', mode='eval')
        new_tree = RewriteName(Map_var).visit(for_node)
        return new_tree
    except:
        traceback.print_exc()
        return None
        # if None in new_node_list:
        #     print("come here")
        #     return None

        # print(ast.unparse(new_tree))

if __name__ == '__main__':
    code='''
# a[0]
# a[1]
# a[2]

# a[3][1:3]
# a[3][3]
# a[5][0][1]
# # a[3][5:][2][2]
# # a[3][1]
# # a[3][5:][2][2]
# # a[4][3]
# # a[2]
# # a[3][1:3]
# # a[3][5:][2][2]
# for quad in quads:
#     upper_edge_len = np.linalg.norm(quad[0] - quad[1])
#     upper_edge_list.append(upper_edge_len)
# for quad in poly_quads:
#     quad_h = (np.linalg.norm(quad[0] - quad[3]) + np.linalg.norm(quad[2] - quad[1])) / 2.0
#     height_list.append(quad_h)
# 这个不可以
# for e in mitie_entities:
#     if len(e[0]):
#         start = tokens[e[0][0]].start
#         end = tokens[e[0][-1]].end
#         entities.append({ENTITY_ATTRIBUTE_TYPE: e[1], ENTITY_ATTRIBUTE_VALUE: text[start:end], ENTITY_ATTRIBUTE_START: start, ENTITY_ATTRIBUTE_END: end, ENTITY_ATTRIBUTE_CONFIDENCE: None})

# for joint in annos:
#     if x <= joint[0][0] < x + target_size[0] and y <= joint[0][1] < y + target_size[1]:
#         break
# for point in joint:
#     if point[0] < -10 or point[1] < -10:
#         adjust_joint.append((-1000, -1000))
#         continue
#     (new_x, new_y) = (point[0] - x, point[1] - y)
#     if new_x > w - 1 or new_y > h - 1:
#         adjust_joint.append((-1000, -1000))
#         continue
#     adjust_joint.append((new_x, new_y))
# 这个不可以（有val[1]还有val[1][0],val[1][0]）
for val in sorted(npz.items()):
    logging.info('  Loading %s in %s' % (str(val[1][0].shape), val[0]))
    logging.info('  Loading %s in %s' % (str(val[1][1].shape), val[0]))
    weights.extend(val[1])
    if len(model.all_weights) == len(weights):
        break
'''

    tree=ast.parse(code)
    for_node=None
    node_list=[]
    for node in ast.walk(tree):
        if isinstance(node,ast.Subscript):
            print("node: ",ast.unparse(node))
            node_list.append(node)
        if isinstance(node,ast.For):
            for_node=node
    print(node_list)
    node_list = node_list[:2]#+node_list[5:6]
    Map_var = dict()
    transform_for_node_var_unpack(for_node, node_list,Map_var)
    '''
    test all code1
    print(sort_node_list(node_list))
    node_list=sort_node_list(node_list)
    new_node_list=[]
    Map_var=dict()
    transform(node_list, new_node_list, 1,Map_var)
    print("Map_var: ",Map_var)
    print("node_list: ",new_node_list)
    tuple = ast.Tuple()
    tuple.elts = []
    build_elts(new_node_list, tuple.elts)
    print("tuple: ",tuple,ast.unparse(tuple))
    new_code_tree=for_node
    for a in ast.walk(for_node):
        if a in Map_var:
            a=Map_var[a]
            print("yes: ",a)
    print("new_for: ",ast.unparse(for_node))
    for_node.target=tuple
    new_tree = RewriteName().visit(tree)
    print(ast.unparse(new_tree))
    '''

    '''
    test sort list
    node_list = [node_list[0], node_list[1], node_list[2]]
    node_list=sort_node_list(node_list)
    
    print_list(e_list)
    # for e,code1 in node_list:
    #     print(e,code1)
    '''

    # print(ast.unparse(build_target(node_list)))
    # 对找到的代码片段不断进行转换直至不可以转换
    # code_list=[
    #     ["user_check = self.run_function('file.get_user', [check])","mode_check = self.run_function('file.get_mode', [check])"],
    #     ["name = obj.find('name').text.strip()","bbox = obj.find('bndbox')"],
    #     ["self._tmp_path = tmp_path","self._config = config","self.location = str(tmp_path)"]]
    # # while 1:
    # save_complicated_code_dir = util.data_root + "complicated_code_dir/var_unpack_func_call_only_same_dengcha_subscript_complicated_pkl/"
    # for file_name in os.listdir(save_complicated_code_dir):
    #     complicate_code = util.load_pkl(save_complicated_code_dir, file_name[:-4])
    #     for each_file in complicate_code:
    #         # for code_list, file_path,file_html in each_file:
    #         #
    #         #     print("count: ",code_list)
    #         code_map = each_file[0]
    #         for node, item in code_map.items():
    #             print(">>>>>>>old code1: ", ast.unparse(node))
    #             transform_var_unpack_call(node, item)
    #
    #
    # test_save_complicated_code_dir = util.data_root + "test_complicated_code_dir/"
    # file_name = "var_unpack_func_call_only_same_dengcha_subscript_complicated"
    # map_code=util.load_pkl(test_save_complicated_code_dir,file_name)
    # for node, item in map_code.items():
    #     print(">>>>>>>old code1: ", ast.unparse(node))
    #     transform_var_unpack_call(node,item)
















